<?php

namespace Ipol\Catapulto\Wordpress;

use Ipol\Catapulto\Wordpress\Controller\AuthController;
use Ipol\Catapulto\Wordpress\DB\CartVariantsDB;
use Ipol\Catapulto\Wordpress\DB\OperatorsDB;
use Ipol\Catapulto\Wordpress\Tools\Variables;
use Ipol\Catapulto\Wordpress\Tools\VariantStore;

class CatapultoShippingMethod extends \WC_Shipping_Method
{

    private $operators = [];

    public function __construct($instance_id = 0)
    {
        parent::__construct($instance_id);

        $this->id = CatapultoPlugin::PLUGIN_ID;
        $this->instance_id = $instance_id;
        $this->title = Variables::getInstance()->getPluginTitle();
        $this->method_title = Variables::getInstance()->getPluginTitle();
        $this->method_description = Variables::getInstance()->getPluginDescription();
        $this->supports = [
            'settings', 'shipping-zones'/*,'instance-settings'*/
        ];

        $this->enabled = 'yes';

        $operatorsDB = new OperatorsDB();
        $this->operators = $operatorsDB->getAllByColumn('op_enabled','Y');

        $this->init_settings();
        $this->init_form_fields();
        add_action("woocommerce_update_options_shipping_{$this->id}", [$this, 'process_admin_options']);
    }

    private function renderSettingsSection(string $sectionCode): string
    {
        $result = '';
        foreach (self::getSettingFields()[$sectionCode] as $key => $field) {
            if (isset($field['render']) && ($field['render'] == false)) continue;
            $result .= $this->renderSettingBlock($key, $field);
        }
        return $result;
    }
    private function renderHiddenSection(string $sectionCode): string
    {
        $result = '';
        foreach (self::getSettingFields()[$sectionCode] as $key => $field) {
            if (isset($field['render']) && ($field['render'] == false)) continue;

            switch ($field['type']) {
                case 'text':
                    $result .= $this->generateHiddenInput($key, $field);
                    break;
                case 'checkbox':
                    $result .= $this->generateHiddenCheckbox($key, $field);
                    break;
                case 'select':
                    $result .= $this->generateHiddenSelect($key, $field);
                    break;
                case 'multiselect':
                    $result .= $this->generateHiddenMultiselect($key, $field);
                    break;
                case 'textarea':
                    $result .= $this->generateHiddenTextarea($key, $field);
                    break;
            }

        }
        return $result;
    }

    private function renderSettingBlock(string $sectionKey, array $sectionBlock):string
    {
        switch ($sectionBlock['type']) {
            case 'title':
                return $this->generateTitleHtml($sectionKey, $sectionBlock);
            case 'text':
                return $this->generateTextHtml($sectionKey, $sectionBlock);
            case 'checkbox':
                return $this->generateCheckboxHtml($sectionKey, $sectionBlock);
            case 'select':
                return $this->generateSelectHtml($sectionKey, $sectionBlock);
            case 'multiselect':
                return $this->generateMultiselectHtml($sectionKey, $sectionBlock);
            case 'textarea':
                return $this->generateTextareaHtml($sectionKey, $sectionBlock);
        }
        return '';
    }

    public function admin_options() {
        echo $this->generate_settings_html( $this->get_form_fields(), false );
    }

    public function generate_settings_html($form_fields = array(), $echo = true)
    {
        $pGateways = [];
        $GW = WC()->payment_gateways();
        foreach ($GW->payment_gateways as $gateway) {
            if ($gateway->enabled=='yes') {
                if ($gateway->title!='') $pGateways[$gateway->id]=$gateway->title;
                else $pGateways[$gateway->id]=$gateway->method_title;
            }
        }
        $form_fields['payNal']['options'] = $pGateways;
        $form_fields['payCard']['options'] = $pGateways;

        $statusesFields = '';
        $statusesFieldsHidden = '';
        foreach (self::getCatapultoStatuses() as $code => $status) {
            $statusesFields .= $this->renderSettingBlock('fp_'.$code, $form_fields['fp_'.$code]);
            $statusesFieldsHidden .= $this->generateHiddenSelect('fp_'.$code, $form_fields['fp_'.$code]);
        }

        $operators = <<<HTML
            <tr><th scope="row" class="titledesc"></th><td class="forminp">
                <h3>Операторы доставки, способ отправки, код терминала по умолчанию</h3>
                <div class="ctpt_operators_h">
                    <p class="free_dlv">Бесплатная доставка</p>
                    <p class="dlv_type">Способ отправки</p>
                    <p class="term_code">Код терминала по умолчанию</p>
                </div>
            </td></tr>
        HTML;
        $operatorsHidden = '';
        foreach ($this->operators as $operator) {
            $code = $operator['operator_id'];

            $operators .= '<tr valign="top">
                <th scope="row" class="titledesc">
                    <label for="ctpt_op_'.$code.'">'.$operator['operator_display'].'</label>
                </th>
                <td class="forminp ctpt_op_element">
                    <fieldset>
                        '
                        . '<div class="ctpt_check_wr">' . $this->generateCheckboxOnly('op_free_'.$code, $form_fields['op_free_'.$code]) . '</div>'
                        . $this->generateSelectOnly('op_type_'.$code, $form_fields['op_type_'.$code])
                        . $this->generateTextOnly('op_code_'.$code, $form_fields['op_code_'.$code])
                        . '
                    </fieldset>
                </td>
            </tr>';
            $operatorsHidden .=
                $this->generateHiddenCheckbox('op_free_'.$code, $form_fields['op_free_'.$code]) .
                $this->generateHiddenSelect('op_type_'.$code, $form_fields['op_type_'.$code]) .
                $this->generateHiddenInput('op_code_'.$code, $form_fields['op_code_'.$code])
            ;
        }

        if (
            empty($this->settings)
            || empty($this->settings['apikey'])
        ) return '<table class="form-table"><input type="hidden" name="ctptIsAuth" value="Y" />' .
            $this->renderSettingsSection('auth') .
            $statusesFieldsHidden .
            $operatorsHidden .
            $this->renderHiddenSection('main') .
            $this->renderHiddenSection('terminals') .
            $this->renderHiddenSection('gabs') .
            $this->renderHiddenSection('dlvsettings') .
            $this->renderHiddenSection('statuses') .
            $this->renderHiddenSection('widget') .
            $this->renderHiddenSection('paysystems') .
            $this->generateHiddenMultiselect('payNal', $form_fields['payNal']) .
            $this->generateHiddenMultiselect('payCard', $form_fields['payCard']) .
            $this->renderHiddenSection('service') .
            '</table>';

        $operators .= '
            <tr valign="top">
                <th scope="row" class="titledesc">
                    <label for="ctpt_op_btns">Установить отправку для всех курьерских служб:</label>
                </th>
                <td class="forminp ctpt_op_element">
                    <fieldset>
                        <button id="ctpt_op_setalldoors" class="components-button is-primary">От двери</button>
                        <button id="ctpt_op_setallwh" class="components-button is-primary">На склад</button>
                    </fieldset>
                </td>
            </tr>
        ';

        $nonce = wp_create_nonce('wp_rest');

        $html = '
            <nav class="nav-tab-wrapper woo-nav-tab-wrapper" data-tabs-content-level="2">
                <a href="#" class="nav-tab ctpt-tab nav-tab-active" data-tab-content-id="ctpt_settings_main">Основные</a>
                <a href="#" class="nav-tab ctpt-tab" data-tab-content-id="ctpt_settings_terminals">Терминалы</a>
                <a href="#" class="nav-tab ctpt-tab" data-tab-content-id="ctpt_settings_gabs">Габариты и настройки доставки</a>
                <a href="#" class="nav-tab ctpt-tab" data-tab-content-id="ctpt_settings_statuses">Синхронизация статусов</a>
                <a href="#" class="nav-tab ctpt-tab" data-tab-content-id="ctpt_settings_widget">Виджет</a>
                <a href="#" class="nav-tab ctpt-tab" data-tab-content-id="ctpt_settings_paysystems">Платежные системы</a>
                <a href="#" class="nav-tab ctpt-tab" data-tab-content-id="ctpt_settings_service">Служебные настройки</a>
            </nav>
            <div class="tab-wrapper">
                <div class="ctpt-tab-content-2" id="ctpt_settings_main">
                    <div class="ctpt_tbl">
                        <p style="font-size: 20px"><b>Ваш ключ: </b>' . \mb_substr($this->settings['apikey'], 0, 30) . '...</p>
                    </div>
                    <div class="ctpt_tbl ctpt_opsyncbl">
                        <button id="ctpt_auth_logout" class="components-button is-primary" type="submit" name="save" value="ctpt_logout">Удалить ключ</button>
                        <button id="ctpt_operators_upd" class="components-button is-primary" k="'.$nonce.'">Обновить операторы</button>
                        <p id="ctpt_operators_upd_txt"></p>
                    </div>
                    <table class="form-table">
                        '.$this->renderSettingsSection('main').'
                    </table>
                </div>
                <div class="ctpt-tab-content-2" id="ctpt_settings_terminals" style="display:none;">
                    <h3>Способ отправки и терминалы отправки по умолчанию</h3>
                    <table class="form-table">
                    '.$this->renderSettingsSection('terminals') . $operators . '
                    </table>
                </div>
                <div class="ctpt-tab-content-2" id="ctpt_settings_gabs" style="display:none;">
                    <h3>Габариты по умолчанию</h3>
                    <table class="form-table">'.$this->renderSettingsSection('gabs').'</table>
                    
                    <h3>Настройки доставки</h3>
                    <table class="form-table">'.$this->renderSettingsSection('dlvsettings').'</table>
                </div>
                <div class="ctpt-tab-content-2" id="ctpt_settings_statuses" style="display:none;">
                    <h3>Синхронизация статусов</h3>
                    <table class="form-table">'
                        . $statusesFields
                        . $this->renderSettingsSection('statuses')
                    .'</table>
                </div>
                <div class="ctpt-tab-content-2" id="ctpt_settings_widget" style="display:none;">
                    <h3>Настройки виджета</h3>
                    <table class="form-table">'.$this->renderSettingsSection('widget').'</table>
                </div>
                <div class="ctpt-tab-content-2" id="ctpt_settings_paysystems" style="display:none;">
                    <h3>Настройки соответствия платежных систем</h3>
                    <h4>Пояснение к платежным системам</h4>
                    <p>Настройка предназначена для корректности учета способа оплаты, выбранного покупателем (клиентом сайта).</p>
                    <p>Необходимо указать какие именно платежные системы, установленные на сайте, считаются оплатой наличными, а какие - оплатой картой при получении клиентом сайта заказа у курьера или на пункте самовывоза. Иными словами, при выборе каких платежных систем считается, что оплата заказа производится наложенным платежом при получении.</p>
                    <p>Если имеются платежные системы, не подразумевающие наложенный платеж: оплата пластиковой картой на сайте, выставление счета, банковский перевод и т.д. - не отмечайте их в этих селекторах! Все это предоплатные платежные системы, подразумевающие оплату клиентом сайта напрямую интернет-магазину, без приема оплаты заказа наложенным платежом.</p>
                    <table class="form-table">'
                        . $this->renderSettingsSection('paysystems')
                        . $this->renderSettingBlock('payNal', $form_fields['payNal'])
                        . $this->renderSettingBlock('payCard', $form_fields['payCard'])
                    . '</table>
                </div>
                <div class="ctpt-tab-content-2" id="ctpt_settings_service" style="display:none;">
                    <h3>Сервисные свойства</h3>
                    <table class="form-table">'.$this->renderSettingsSection('service').'</table>
                </div>
            </div>
        ';

        return $html;
    }

    private function generateTitleHtml( $key, $data ) {
        $field_key = $this->get_field_key( $key );
        $defaults  = array(
            'title' => '',
            'class' => '',
        );

        $data = wp_parse_args( $data, $defaults );

        return '
            <h3 class="wc-settings-sub-title '.esc_attr( $data['class'] ).'" id="'.esc_attr( $field_key ).'">'.wp_kses_post( $data['title'] ).'</h3>
            '. (( !empty( $data['description'] ) ) ? '<p>'.wp_kses_post( $data['description'] ).'</p>' : '') .'
        ';
    }

    /*Hidden blocks*/
    private function generateHiddenInput($key, $data)
    {
        $field_key = $this->get_field_key( $key );
        $defaults  = [
            'title'             => '',
            'disabled'          => false,
            'class'             => '',
            'css'               => '',
            'placeholder'       => '',
            'type'              => 'text',
            'desc_tip'          => false,
            'description'       => '',
            'custom_attributes' => [],
        ];

        $data = wp_parse_args( $data, $defaults );

        return '<input class="ctpt_inputhidden" type="text" name="'.esc_attr( $field_key ).'" id="'.esc_attr( $field_key ).'" value="'.esc_attr( $this->get_option( $key ) ).'" '.$this->get_custom_attribute_html( $data ).' />';
    }
    private function generateHiddenCheckbox( $key, $data ) {
        $field_key = $this->get_field_key( $key );
        $defaults  = [
            'title'             => '',
            'label'             => '',
            'disabled'          => false,
            'class'             => '',
            'css'               => '',
            'type'              => 'text',
            'desc_tip'          => false,
            'description'       => '',
            'custom_attributes' => [],
        ];

        $data = wp_parse_args( $data, $defaults );

        if ( ! $data['label'] ) {
            $data['label'] = $data['title'];
        }

        return '<input class="ctpt_inputhidden" type="checkbox" name="'.esc_attr( $field_key ).'" value="1" '.checked( $this->get_option( $key ), 'yes', false ).' '.$this->get_custom_attribute_html( $data ).' />';
    }
    private function generateHiddenSelect( $key, $data ) {
        $field_key = $this->get_field_key( $key );
        $defaults  = [
            'title'             => '',
            'disabled'          => false,
            'class'             => '',
            'css'               => '',
            'placeholder'       => '',
            'type'              => 'text',
            'desc_tip'          => false,
            'description'       => '',
            'custom_attributes' => [],
            'options'           => [],
        ];

        $data  = wp_parse_args( $data, $defaults );
        $value = $this->get_option( $key );

        $optionsHtml = '';
        foreach ( (array) $data['options'] as $option_key => $option_value ) {
            if ( is_array( $option_value ) ) {
                $opts = '';
                foreach ( $option_value as $option_key_inner => $option_value_inner ) {
                    $opts .= '<option value="'.esc_attr( $option_key_inner ).'" '.selected( (string) $option_key_inner, esc_attr( $value ), false ).'>'.esc_html( $option_value_inner ).'</option>';
                }
                $optionsHtml .= '<optgroup label="'.esc_attr( $option_key ).'">'.$opts.'</optgroup>';
            } else {
                $optionsHtml .= '<option value="'.esc_attr( $option_key ).'" '.selected( (string) $option_key, esc_attr( $value ), false ).'>'.esc_html( $option_value ).'</option>';
            }
        }

        return '<select class="ctpt_inputhidden" name="'.esc_attr( $field_key ).'" '.$this->get_custom_attribute_html( $data ).'>'.$optionsHtml.'</select>';
    }
    private function generateHiddenMultiselect( $key, $data ) {
        $field_key = $this->get_field_key( $key );
        $defaults  = [
            'title'             => '',
            'disabled'          => false,
            'class'             => '',
            'css'               => '',
            'placeholder'       => '',
            'type'              => 'text',
            'desc_tip'          => false,
            'description'       => '',
            'custom_attributes' => [],
            'select_buttons'    => false,
            'options'           => [],
        ];

        $data  = wp_parse_args( $data, $defaults );
        $value = (array) $this->get_option( $key, [] );

        $options = '';
        foreach ((array) $data['options'] as $option_key => $option_value) {
            if ( is_array( $option_value ) ) {
                $optVals = '';
                foreach ( $option_value as $option_key_inner => $option_value_inner ) {
                    $optVals .= '<option value="'.esc_attr( $option_key_inner ).'" '.selected( in_array( (string) $option_key_inner, $value, true, false ), true ).'>'.esc_html( $option_value_inner ).'</option>';
                }
                $options .= '<optgroup label="'.esc_attr( $option_key ).'">'.$optVals.'</optgroup>';
            } else {
                $options .= '<option value="'.esc_attr( $option_key ).'" '.selected( in_array( (string) $option_key, $value, true ), true, false ).'>'.esc_html( $option_value ).'</option>';
            }
        }

        return '<select class="ctpt_inputhidden" multiple="multiple" class="multiselect '.esc_attr( $data['class'] ).'" name="'.esc_attr( $field_key ).'[]" id="'.esc_attr( $field_key ).'" '.$this->get_custom_attribute_html( $data ).'>'.$options.'</select>';
    }
    private function generateHiddenTextarea( $key, $data ) {
        $field_key = $this->get_field_key( $key );
        $defaults  = array(
            'title'             => '',
            'disabled'          => false,
            'class'             => '',
            'css'               => '',
            'placeholder'       => '',
            'type'              => 'text',
            'desc_tip'          => false,
            'description'       => '',
            'custom_attributes' => array(),
        );

        $data = wp_parse_args( $data, $defaults );

        $idEl = esc_attr( $field_key );
        $inputType = esc_attr( $data['type'] );
        $currentValue = esc_textarea( $this->get_option( $key ) );

        return <<<HTML
            <textarea rows="3" cols="20" type="$inputType" name="$idEl" id="$idEl" style="display:none !important;">$currentValue</textarea>
        HTML;
    }
    /* /Hidden blocks*/

    private function generateTextHtml($key, $data)
    {
        $field_key = $this->get_field_key( $key );
        $defaults  = [
            'title'             => '',
            'disabled'          => false,
            'class'             => '',
            'css'               => '',
            'placeholder'       => '',
            'type'              => 'text',
            'desc_tip'          => false,
            'description'       => '',
            'custom_attributes' => [],
        ];

        $data = wp_parse_args( $data, $defaults );

        return '
        <tr valign="top">
            <th scope="row" class="titledesc">
                <label for="'.esc_attr( $field_key ).'">'.wp_kses_post( $data['title'] ).' '.$this->get_tooltip_html( $data ).'</label>
            </th>
            <td class="forminp">
                <fieldset>
                    <legend class="screen-reader-text"><span>'.wp_kses_post( $data['title'] ).'</span></legend>
                    <input class="input-text regular-input '.esc_attr( $data['class'] ).'" type="'.esc_attr( $data['type'] ).'" name="'.esc_attr( $field_key ).'" id="'.esc_attr( $field_key ).'" style="'.esc_attr( $data['css'] ).'" value="'.esc_attr( $this->get_option( $key ) ).'" placeholder="'.esc_attr( $data['placeholder'] ).'" '.disabled( $data['disabled'], true ).' '.$this->get_custom_attribute_html( $data ).' />
                    '.$this->get_description_html( $data ).'
                </fieldset>
            </td>
        </tr>
        ';
    }
    private function generateTextOnly($key, $data)
    {
        $field_key = $this->get_field_key( $key );
        $defaults  = [
            'title'             => '',
            'disabled'          => false,
            'class'             => '',
            'css'               => '',
            'placeholder'       => '',
            'type'              => 'text',
            'desc_tip'          => false,
            'description'       => '',
            'custom_attributes' => [],
        ];

        $data = wp_parse_args( $data, $defaults );

        return '<input class="input-text regular-input '.esc_attr( $data['class'] ).'" type="'.esc_attr( $data['type'] ).'" name="'.esc_attr( $field_key ).'" id="'.esc_attr( $field_key ).'" style="'.esc_attr( $data['css'] ).'" value="'.esc_attr( $this->get_option( $key ) ).'" placeholder="'.esc_attr( $data['placeholder'] ).'" '.disabled( $data['disabled'], true ).' '.$this->get_custom_attribute_html( $data ).' />';
    }

    private function generateCheckboxHtml( $key, $data ) {
        $field_key = $this->get_field_key( $key );
        $defaults  = [
            'title'             => '',
            'label'             => '',
            'disabled'          => false,
            'class'             => '',
            'css'               => '',
            'type'              => 'text',
            'desc_tip'          => false,
            'description'       => '',
            'custom_attributes' => [],
        ];

        $data = wp_parse_args( $data, $defaults );

        if ( ! $data['label'] ) {
            $data['label'] = $data['title'];
        }

        return '<tr valign="top">
            <th scope="row" class="titledesc">
                <label for="'.esc_attr( $field_key ).'">'.wp_kses_post( $data['title'] ).' '.$this->get_tooltip_html( $data ).'</label>
            </th>
            <td class="forminp">
                <fieldset>
                    <legend class="screen-reader-text"><span>'.wp_kses_post( $data['title'] ).'</span></legend>
                    <label for="'.esc_attr( $field_key ).'">
                        <input '.disabled( $data['disabled'], true ).' class="'.esc_attr( $data['class'] ).'" type="checkbox" name="'.esc_attr( $field_key ).'" id="'.esc_attr( $field_key ).'" style="'.esc_attr( $data['css'] ).'" value="1" '.checked( $this->get_option( $key ), 'yes', false ).' '.$this->get_custom_attribute_html( $data ).' /> '.wp_kses_post( $data['label'] ).'</label><br/>
                    '.$this->get_description_html( $data ).'
                </fieldset>
            </td>
        </tr>';
    }
    private function generateCheckboxOnly( $key, $data ) {
        $field_key = $this->get_field_key( $key );
        $defaults  = [
            'title'             => '',
            'label'             => '',
            'disabled'          => false,
            'class'             => '',
            'css'               => '',
            'type'              => 'text',
            'desc_tip'          => false,
            'description'       => '',
            'custom_attributes' => [],
        ];

        $data = wp_parse_args( $data, $defaults );

        if ( ! $data['label'] ) {
            $data['label'] = $data['title'];
        }

        return '<input class="'.esc_attr( $data['class'] ).'" type="checkbox" name="'.esc_attr( $field_key ).'" id="'.esc_attr( $field_key ).'" value="1" '.checked( $this->get_option( $key ), 'yes', false ).' '.$this->get_custom_attribute_html( $data ).' />';
    }

    private function generateSelectHtml( $key, $data ) {
        $field_key = $this->get_field_key( $key );
        $defaults  = [
            'title'             => '',
            'disabled'          => false,
            'class'             => '',
            'css'               => '',
            'placeholder'       => '',
            'type'              => 'text',
            'desc_tip'          => false,
            'description'       => '',
            'custom_attributes' => [],
            'options'           => [],
        ];

        $data  = wp_parse_args( $data, $defaults );
        $value = $this->get_option( $key );

        $optionsHtml = '';
        foreach ( (array) $data['options'] as $option_key => $option_value ) {
            if ( is_array( $option_value ) ) {
                $opts = '';
                foreach ( $option_value as $option_key_inner => $option_value_inner ) {
                    $opts .= '<option value="'.esc_attr( $option_key_inner ).'" '.selected( (string) $option_key_inner, esc_attr( $value ), false ).'>'.esc_html( $option_value_inner ).'</option>';
                }
                $optionsHtml .= '<optgroup label="'.esc_attr( $option_key ).'">'.$opts.'</optgroup>';
            } else {
                $optionsHtml .= '<option value="'.esc_attr( $option_key ).'" '.selected( (string) $option_key, esc_attr( $value ), false ).'>'.esc_html( $option_value ).'</option>';
            }
        }

        return '<tr valign="top">
            <th scope="row" class="titledesc">
                <label for="'.esc_attr( $field_key ).'">'.wp_kses_post( $data['title'] ).' '.$this->get_tooltip_html( $data ).'</label>
            </th>
            <td class="forminp">
                <fieldset>
                    <legend class="screen-reader-text"><span>'.wp_kses_post( $data['title'] ).'</span></legend>
                    <select class="select '.esc_attr( $data['class'] ).'" name="'.esc_attr( $field_key ).'" id="'.esc_attr( $field_key ).'" style="'.esc_attr( $data['css'] ).'" '.disabled( $data['disabled'], true ).' '.$this->get_custom_attribute_html( $data ).'>'.$optionsHtml.'</select>
                    '.$this->get_description_html( $data ).'
                </fieldset>
            </td>
        </tr>';
    }
    private function generateSelectOnly( $key, $data ) {
        $field_key = $this->get_field_key( $key );
        $defaults  = [
            'title'             => '',
            'disabled'          => false,
            'class'             => '',
            'css'               => '',
            'placeholder'       => '',
            'type'              => 'text',
            'desc_tip'          => false,
            'description'       => '',
            'custom_attributes' => [],
            'options'           => [],
        ];

        $data  = wp_parse_args( $data, $defaults );
        $value = $this->get_option( $key );

        $optionsHtml = '';
        foreach ( (array) $data['options'] as $option_key => $option_value ) {
            if ( is_array( $option_value ) ) {
                $opts = '';
                foreach ( $option_value as $option_key_inner => $option_value_inner ) {
                    $opts .= '<option value="'.esc_attr( $option_key_inner ).'" '.selected( (string) $option_key_inner, esc_attr( $value ), false ).'>'.esc_html( $option_value_inner ).'</option>';
                }
                $optionsHtml .= '<optgroup label="'.esc_attr( $option_key ).'">'.$opts.'</optgroup>';
            } else {
                $optionsHtml .= '<option value="'.esc_attr( $option_key ).'" '.selected( (string) $option_key, esc_attr( $value ), false ).'>'.esc_html( $option_value ).'</option>';
            }
        }

        return '<select class="select '.esc_attr( $data['class'] ).'" name="'.esc_attr( $field_key ).'" id="'.esc_attr( $field_key ).'" style="'.esc_attr( $data['css'] ).'" '.disabled( $data['disabled'], true ).' '.$this->get_custom_attribute_html( $data ).'>'.$optionsHtml.'</select>';
    }

    private function generateMultiselectHtml( $key, $data ) {
        $field_key = $this->get_field_key( $key );
        $defaults  = [
            'title'             => '',
            'disabled'          => false,
            'class'             => '',
            'css'               => '',
            'placeholder'       => '',
            'type'              => 'text',
            'desc_tip'          => false,
            'description'       => '',
            'custom_attributes' => [],
            'select_buttons'    => false,
            'options'           => [],
        ];

        $data  = wp_parse_args( $data, $defaults );
        $value = (array) $this->get_option( $key, [] );

        $options = '';
        foreach ((array) $data['options'] as $option_key => $option_value) {
            if ( is_array( $option_value ) ) {
                $optVals = '';
                foreach ( $option_value as $option_key_inner => $option_value_inner ) {
                    $optVals .= '<option value="'.esc_attr( $option_key_inner ).'" '.selected( in_array( (string) $option_key_inner, $value, true, false ), true ).'>'.esc_html( $option_value_inner ).'</option>';
                }
                $options .= '<optgroup label="'.esc_attr( $option_key ).'">'.$optVals.'</optgroup>';
            } else {
                $options .= '<option value="'.esc_attr( $option_key ).'" '.selected( in_array( (string) $option_key, $value, true ), true, false ).'>'.esc_html( $option_value ).'</option>';
            }
        }

        $selectBtn = '';
        if ( $data['select_buttons'] ) {
            $selectBtn = '<br/><a class="select_all button" href="#">'.esc_html_e( 'Select all', 'woocommerce' ).'</a> <a class="select_none button" href="#">'.esc_html_e( 'Select none', 'woocommerce' ).'</a>';
        }

        return '<tr valign="top">
            <th scope="row" class="titledesc">
                <label for="'.esc_attr( $field_key ).'">'.wp_kses_post( $data['title'] ).' '.$this->get_tooltip_html( $data ).'</label>
            </th>
            <td class="forminp">
                <fieldset>
                    <legend class="screen-reader-text"><span>'.wp_kses_post( $data['title'] ).'</span></legend>
                    <select multiple="multiple" class="multiselect '.esc_attr( $data['class'] ).'" name="'.esc_attr( $field_key ).'[]" id="'.esc_attr( $field_key ).'" style="'.esc_attr( $data['css'] ).'" '.disabled( $data['disabled'], true ).' '.$this->get_custom_attribute_html( $data ).'>'.$options.'</select>
                    '.$this->get_description_html( $data ) . $selectBtn .'
                </fieldset>
            </td>
        </tr>';
    }

    private function generateTextareaHtml( $key, $data ) {
        $field_key = $this->get_field_key( $key );
        $defaults  = array(
            'title'             => '',
            'disabled'          => false,
            'class'             => '',
            'css'               => '',
            'placeholder'       => '',
            'type'              => 'text',
            'desc_tip'          => false,
            'description'       => '',
            'custom_attributes' => array(),
        );

        $data = wp_parse_args( $data, $defaults );

        $idEl = esc_attr( $field_key );
        $title = wp_kses_post( $data['title'] );
        $tooltip = $this->get_tooltip_html( $data );
        $class = esc_attr( $data['class'] );
        $inputType = esc_attr( $data['type'] );
        $styles = esc_attr( $data['css'] );
        $placeholder = esc_attr( $data['placeholder'] );
        $disabled = disabled( $data['disabled'], true );
        $customAttributes = $this->get_custom_attribute_html( $data );
        $currentValue = esc_textarea( $this->get_option( $key ) );
        $description = $this->get_description_html( $data );

        return <<<HTML
        <tr valign="top">
            <th scope="row" class="titledesc">
                <label for="$idEl">$title $tooltip</label>
            </th>
            <td class="forminp">
                <fieldset>
                    <legend class="screen-reader-text"><span>$title</span></legend>
                    <textarea rows="3" cols="20" class="input-text wide-input $class" type="$inputType" name="$idEl" id="$idEl" style="$styles" placeholder="$placeholder" $disabled $customAttributes>$currentValue</textarea>
                    $description
                </fieldset>
            </td>
        </tr>
        HTML;
    }

    public function init_form_fields(): void
    {
        $this->form_fields = [];
        foreach (self::getSettingFields() as $key => $fields) $this->form_fields = array_merge($this->form_fields, $fields);
        //add Order Statuses....
        if($wcStatuses =  wc_get_order_statuses()) {
            $options = array_merge(['not_set' => 'Не установлено'], $wcStatuses);
            foreach (self::getCatapultoStatuses() as $code => $status)
                $this->form_fields['fp_' . $code] = [
                    'title' => $status,
                    'type' => 'select',
                    'desc_tip' => false,
                    'default' => 'not_set',
                    'options' => $options
                ];
        }

        //Operators...
        foreach ($this->operators as $operator) {
            $this->form_fields['op_type_' . $operator['operator_id']] = [
                'title'   => '',
                'type'    => 'select',
                'default' => 'door',
                'description' => '',
                'desc_tip'    => false,
                'options'=> [
                    'door' => 'От двери',
                    'warehouse' => 'На склад',
                ]
            ];
            $this->form_fields['op_code_' . $operator['operator_id']] = [
                'title' => '',
                'description' => '',
                'desc_tip' => false,
                'type' => 'text',
                'default' => '',
            ];
            $this->form_fields['op_free_' . $operator['operator_id']] = [
                'title' => '',
                'description' => '',
                'desc_tip' => false,
                'type' => 'checkbox',
                'default' => 'no',
            ];
        }
    }

    public function validate_text_field($key, $value)
    {
        $postData = $this->get_post_data();
        $isAuthMode = isset($postData['ctptIsAuth']) && ($postData['ctptIsAuth'] === 'Y');

        $value = parent::validate_text_field($key, $value);
        switch ($key) {
            case 'apikey':
                if ($postData['save'] === 'ctpt_logout') return '';
                if (!$isAuthMode) return $this->settings[$key];
                if (empty($value)) {
                    \WC_Admin_Settings::add_error('Необходимо указать ключ доступа к сервису Catapulto.');
                    return false;
                }
                $isTestMode = ($this->getSimpleFieldValue('isTest') === '1') || ($this->getSimpleFieldValue('isTest') === 'yes');
                $customApiUrl = $this->getSimpleFieldValue('customApiUrl');
                if (!$isTestMode) $customApiUrl = '';
                $authController = new AuthController($value, $customApiUrl);
                $checkAuth = $authController->checkAuth();
                if (!$checkAuth) {
                    \WC_Admin_Settings::add_error('Ключ авторизации неверен. Уточните актуальный ключ доступа у Catapulto. ' . $authController->getErrorsMessageByOneString());
                    return false;
                }
                break;
            case 'isTest':
            case 'customApiUrl':
            case 'customWSUrl':
                if (!$isAuthMode) return $this->settings[$key];
                break;
           /* default:
                return parent::validate_text_field($key, $value);*/
        }

        return $value;
    }

    private function getSimpleFieldValue($key)
    {
        $field_key = $this->get_field_key( $key );
        $post_data = empty( $post_data ) ? $_POST : $post_data; // WPCS: CSRF ok, input var ok.
        return isset( $post_data[ $field_key ] ) ? $post_data[ $field_key ] : null;
    }

    public function calculate_shipping($package = []): void
    {
        parse_str(!empty($_POST['post_data']) ? sanitize_text_field($_POST['post_data']) : '', $post_data);
        $ctptRateId = 0;
        if (isset($post_data['ctpt_selected_rate'])) $ctptRateId = intval($post_data['ctpt_selected_rate']);

        $variantData = VariantStore::getVariantStatic();
        $price = floatval($this->settings['deliveryDefaultPrice']);
        if (!empty($variantData) && isset($variantData['variant'])) $price = $variantData['variant']['price'];

        $wcData = WC()->session->get_session_data();
        $orderId = $wcData['store_api_draft_order'] ?? 0;

        $db = new CartVariantsDB();
        if (empty($variantData)) {
             if ($ctptRateId > 0) {
                $variant = $db->getDataByCtptrate($ctptRateId);
                if ($variant) {
                    $variant = unserialize($variant['variant']);
                    $price = $variant['variant']['price'];
                    $variantData = $variant;
                }
            } else if ($orderId > 0) {
                $variant = $db->getDataByCartID($orderId);
                if ($variant) {
                    $variant = unserialize($variant['variant']);
                    $price = $variant['variant']['price'];
                    $variantData = $variant;
                }
            }
        }

        $this->add_rate([
            'id' => $this->id . ':pickup',
            'label' => $this->get_title(),
            'cost' => $price,
            'meta_data' => $variantData
        ]);
    }

    private static function getSettingFields():array
    {
        return [
            'auth' => [
                'title' => [
                    'title' => 'Авторизация в сервисе Catapulto',
                    'type' => 'title',
                ],
                'apikey' => [
                    'title' => 'Ваш ключ доступа',
                    'description' => 'Для начала работы необходимо авторизоваться в сервисе Catapulto',
                    'desc_tip' => true,
                    'type' => 'text',
                    'custom_attributes' => [
                        'required' => true,
                    ],
                    'default' => '',
                ],
                'isTest' => [
                    'title'   => 'Работа в тестовом режиме',
                    'type'    => 'checkbox',
                    'default' => ''
                ],
                'customApiUrl' => [
                    'title' => 'Произвольный адрес для запросов API',
                    'description' => 'В случае работы в тестовом режиме запросы могут быть отправлены на произвольный адрес API',
                    'desc_tip' => true,
                    'type' => 'text',
                    /*'custom_attributes' => [
                        'required' => true,
                    ],*/
                    'default' => '',
                ],
                'customWSUrl' => [
                    'title' => 'Произвольный адрес для запросов WS',
                    'description' => 'В случае работы в тестовом режиме запросы могут быть отправлены на произвольный адрес WS',
                    'desc_tip' => true,
                    'type' => 'text',
                    /*'custom_attributes' => [
                        'required' => true,
                    ],*/
                    'default' => '',
                ],
            ],
            'main' => [
                'dadataApikey' => [
                    'title' => 'API ключ DaData',
                    'description' => 'Для работы модуля требуется ввести API ключ от сервиса DaData. Пройдите регистрацию в сервисе <a href="https://dadata.ru">dadata.ru</a>, скопируйте API ключ из личного кабинета и вставьте в это поле.',
                    'desc_tip' => true,
                    'type' => 'text',
                    'default' => '',
                ],
                'title' => [
                    'title' => 'Отправитель по умолчанию',
                    'type' => 'title',
                ],
                'senderLocalityId' => [
                    'title' => 'ID города отправителя',
                    'description' => 'ID города из которого вы отправляете заказы. Уточните данную информацию у менеджера Catapulto.',
                    'desc_tip' => true,
                    'type' => 'text',
                    'default' => '',
                ],
                'senderZip' => [
                    'title' => 'Индекс отправителя',
                    'description' => 'Индекс города из которого вы отправляете заказы. Уточните данную информацию у менеджера Catapulto.',
                    'desc_tip' => true,
                    'type' => 'text',
                    'default' => '',
                ],
                'senderCity' => [
                    'title' => 'Город отправителя',
                    'description' => 'Город из которого вы отправляете заказы. Например, "Москва" (без кавычек)',
                    'desc_tip' => true,
                    'type' => 'text',
                    'default' => '',
                ],
                'senderId' => [
                    'title' => 'ID отправителя по умолчанию',
                    'description' => 'ID контакта из адресной книги в личном кабинете Catapulto, который будет выбран отправителем в модуле.<br/> Уточнить нужное значение для этой настройки можно у менеджера Catapulto',
                    'desc_tip' => true,
                    'type' => 'text',
                    'default' => '',
                ],
                'poaEnabled' => [
                    'title'   => 'Включить отправку по доверенности (для "Деловые линии")',
                    'type'    => 'checkbox',
                    'default' => ''
                ],
                'poaFromName' => [
                    'title' => 'ФИО контактного лица',
                    'description' => '',
                    'desc_tip' => false,
                    'type' => 'text',
                    'default' => '',
                ],
                'poaFromPassportSeries' => [
                    'title' => 'Серия паспорта (4 цифры)',
                    'description' => '',
                    'desc_tip' => false,
                    'type' => 'text',
                    'default' => '',
                ],
                'poaFromPassportNumber' => [
                    'title' => 'Номер паспорта (6 цифр)',
                    'description' => '',
                    'desc_tip' => false,
                    'type' => 'text',
                    'default' => '',
                ],
                'poaFromPassportDate' => [
                    'title' => 'Дата выдачи (дд.мм.гггг)',
                    'description' => '',
                    'desc_tip' => false,
                    'type' => 'text',
                    'default' => '',
                ],
                'poaSendReceiverEmail' => [
                    'title' => 'Email для отправки доверенности (не обязательно)',
                    'description' => '',
                    'desc_tip' => false,
                    'type' => 'text',
                    'default' => '',
                ],
            ],
            'terminals'=>[
                'widgetDeliveryFrom' => [
                    'title'   => 'Способ передачи отправлений по умолчанию',
                    'type'    => 'select',
                    'default' => 'door',
                    'description' => '',
                    'desc_tip'    => false,
                    'options'=> [
                        'door' => 'От двери',
                        'warehouse' => 'От склада',
                    ]
                ],
                //Terminals....
                'freeDlvMinCourier' => [
                    'title' => 'Бесплатная курьерская доставка от, руб.',
                    'description' => '',
                    'desc_tip' => false,
                    'type' => 'text',
                    'default' => '0',
                ],
                'freeDlvMinPVZ' => [
                    'title' => 'Бесплатная доставка на ПВЗ от, руб.',
                    'description' => '',
                    'desc_tip' => false,
                    'type' => 'text',
                    'default' => '0',
                ],
            ],
            'gabs'=>[
                'defMode' => [
                    'title'   => 'Рассчитывать средние габариты для',
                    'type'    => 'select',
                    'default' => 'O',
                    'description' => '',
                    'desc_tip'    => false,
                    'options'=> [
                        'O' => 'Заказа',
                        'G' => '1 Товара',
                    ]
                ],
                'defaultWidth'=>[
                    'title' => 'Ширина по умолчанию, мм',
                    'description' => '',
                    'desc_tip' => false,
                    'type' => 'text',
                    'default' => '',
                ],
                'defaultHeight'=>[
                    'title' => 'Высота по умолчанию, мм',
                    'description' => '',
                    'desc_tip' => false,
                    'type' => 'text',
                    'default' => '',
                ],
                'defaultLength'=>[
                    'title' => 'Длина по умолчанию, мм',
                    'description' => '',
                    'desc_tip' => false,
                    'type' => 'text',
                    'default' => '',
                ],
                'defaultWeight'=>[
                    'title' => 'Вес по умолчанию, гр',
                    'description' => '',
                    'desc_tip' => false,
                    'type' => 'text',
                    'default' => '',
                ],
            ],
            'dlvsettings'=>[
                'deliveryDefaultPrice'=>[
                    'title' => 'Стоимость доставки по умолчанию ',
                    'description' => 'Стоимость доставки по умолчанию, которая показывается на странице оформления заказа до момента выбора конкретного способа доставки в виджете.',
                    'desc_tip' => true,
                    'type' => 'text',
                    'default' => '',
                ],
                'termIncrease'=>[
                    'title' => 'Увеличить срок доставки на (дн.)',
                    'description' => 'Увеличивает срок доставки на указанное количество дней.<br/><br/>Используется для учета срока на комплектацию и отправку заказа в службу доставки.',
                    'desc_tip' => true,
                    'type' => 'text',
                    'default' => '',
                ],
                'mindEnsurance'=>[
                    'title'   => 'Включать страхование по умолчанию',
                    'description' => 'Включает страхование при расчете стоимости и при отправке заявки из администратовной части',
                    'desc_tip' => true,
                    'type'    => 'checkbox',
                    'default' => 'yes'
                ],
                'noPVZnoOrder'=>[
                    'title'   => 'Не давать оформить заказ без выбранного в виджете способа доставки',
                    'description' => 'Модуль не даст оформить заказ, если не выбран способ доставки в виджете.',
                    'desc_tip' => true,
                    'type'    => 'checkbox',
                    'default' => 'yes'
                ],
                'needReselect'=>[
                    'title'   => 'Не давать оформить заказ после смены способа оплаты',
                    'description' => 'Модуль не даст оформить заказ, если после выбора способа доставки был поменян способ оплаты заказа. В таком случае будет показана ошибка с просьбой покупателя заново выбрать способ доставки в виджете Catapulto',
                    'desc_tip' => true,
                    'type'    => 'checkbox',
                    'default' => 'yes'
                ],
            ],
            'statuses'=>[
                'markPayed'=>[
                    'title'   => 'Отмечать доставленный заказ выполненным',
                    'description' => 'При получении заказом финального статуса "Отправление выдано покупателю" его можно отметить выполненным (статус для заказа WordPress/WooCommerce). <br/><br/>Опция может быть полезна для автоматизации бизнес-процессов, в случаях когда клиент оплачивает заказ наложенным платежом при получении у курьера, либо на точке самовывоза.',
                    'desc_tip' => true,
                    'type'    => 'checkbox',
                    'default' => 'yes'
                ],
            ],
            'widget'=>[
                'widgetDeliveryTypes' => [
                    'title'   => 'Способы доставки в виджете',
                    'type'    => 'select',
                    'default' => 'All',
                    'description' => 'Ограничение способов доставки в виджете. Данная опция позволяет установить доступными определенные способы доставки в виджете (например: только курьерская доставка).',
                    'desc_tip'    => false,
                    'options'=> [
                        'All' => 'Все',
                        'Pvz' => 'Пункты самовывоза',
                        'Courier' => 'Курьерская доставка',
                    ]
                ],
                /*'enDadataSuggestions'=>[
                    'title'   => 'Подключить подсказки Dadata для свойства ввода адреса',
                    'description' => 'В выбранном свойстве сохранения адреса будут подключены подсказки ввода адреса от Dadata.',
                    'desc_tip' => true,
                    'type'    => 'checkbox',
                    'default' => ''
                ],*/
                'enMapOpenMode' => [
                    'title'   => 'Активная вкладка по умолчанию',
                    'type'    => 'select',
                    'default' => 'courier',
                    'description' => 'Выбранная вкладка будет открываться по умолчанию при расчёте.',
                    'desc_tip'    => false,
                    'options'=> [
                        'courier' => 'Курьером до двери',
                        'map' => 'Пункт выдачи',
                    ]
                ],
                'runWidgetOnStart'=>[
                    'title'   => 'Запускать виджет после выбора службы доставки Catapulto',
                    'description' => 'При активации варианта доставки Catapulto на странице оформления заказа виджет запускается сразу.',
                    'desc_tip' => true,
                    'type'    => 'checkbox',
                    'default' => 'yes'
                ],
                'geoEmptyMessage'=>[
                    'title'=>'Текст ошибки в виджете при поиске местоположений',
                    'description'=>'Текст, указанный здесь, будет добавлен к тексту ошибки в виджете, если введенный адрес в поиске не будет найден.',
                    'desc_tip' => true,
                    'type' => 'textarea',
                ],
            ],
            'paysystems'=>[
                'payNal' => [
                    'title' => 'Оплата наличными (наложенный платеж)',
                    'type' => 'multiselect',
                    'options' => [],
                    'render' => false,
                ],
                'payCard' => [
                    'title' => 'Оплата картой при получении (наложенный платеж)',
                    'type' => 'multiselect',
                    'options' => [],
                    'render' => false,
                ],
            ],
            'service'=>[
                'enLog'=>[
                    'title'   => 'Включение логгирование запросов',
                    'description' => 'Запросы и ответы к API будут записываться в отдельный лог-файл',
                    'desc_tip' => true,
                    'type'    => 'checkbox',
                    'default' => ''
                ],
                'client_notify'=>[
                    'title'   => 'Отправлять уведомления о трек-номере',
                    'description' => 'После отправки заявки и появления ссылки отслеживания заказа покупателю отправляется уведомление о возможности отслеживать заказ с ссылкой на статус заявки.',
                    'desc_tip' => true,
                    'type'    => 'checkbox',
                    'default' => 'yes'
                ],

            ],
        ];
    }

    //Catapulto Statuses
    public static function getCatapultoStatuses()
    {
        return [
            'courier_take' => 'Курьер забирает',
            'on_road' => 'В пути',
            'delivery' => 'На доставке',
            'delivery_problem' => 'Проблема с доставкой',
            'reject' => 'Отмена',
            'are_cleared' => 'Проходит таможенную очистку',
            'created' => 'Создан, идет обработка',
            'forwarding' => 'Переадресация',
            'return_to_sender' => 'Возврат отправителю',
            'completed' => 'Доставлено',
            'return_doc' => 'Возврат документов',
        ];
    }

    //Catapulto Tracking Statuses
    public static function getFinalStatuses():array
    {
        return [
            'reject',
            'rejected',
            'return_to_sender',
            'completed',
            'return_doc'
        ];
    }

    //Others
    public function getAllSettings():array
    {
        return $this->settings;
    }

    public function getApiKey():string
    {
        return $this->settings['apikey'] ?? '';
    }

    public function isTestMode():bool
    {
        return ( $this->settings['isTest'] ?? false ) === 'yes';
    }

    public function isLogMode():bool
    {
        return ( $this->settings['enLog'] ?? false ) === 'yes';
    }

    public function getCustomApiURL():string
    {
        $isTest = $this->isTestMode();
        $apiUrl = $this->settings['customApiUrl'] ?? '';
        return $isTest ? $apiUrl : '';
    }

    public function getDadataKey():string
    {
        return $this->settings['dadataApikey'] ?? '';
    }

    public function getSenderLocId():string
    {
        return $this->settings['senderLocalityId'] ?? '';
    }

    public function getSenderZip():string
    {
        return $this->settings['senderZip'] ?? '';
    }

    public function getSenderCity():string
    {
        return $this->settings['senderCity'] ?? '';
    }

    public function getSenderId():string
    {
        return $this->settings['senderId'] ?? '';
    }

    public function getGabsMode():string
    {
        return $this->settings['defMode'] ?? 'O';
    }

    public function getDefaultWidth():float
    {
        return floatval($this->settings['defaultWidth'] ?? PHP_FLOAT_MAX);
    }
    public function getDefaultHeight():float
    {
        return floatval($this->settings['defaultHeight'] ?? PHP_FLOAT_MAX);
    }
    public function getDefaultLength():float
    {
        return floatval($this->settings['defaultLength'] ?? PHP_FLOAT_MAX);
    }
    public function getDefaultWeight():float
    {
        return floatval($this->settings['defaultWeight'] ?? PHP_FLOAT_MAX);
    }

    public function isMarkDeliveriedOrderIsPaid():bool
    {
        return ($this->settings['markPayed'] ?? false) === 'yes';
    }

    public function isEmailNotidyEnabled():bool
    {
        return ($this->settings['client_notify'] ?? false) === 'yes';
    }

    public function isEnabledEmptyRateOrder():bool
    {
        return !(($this->settings['noPVZnoOrder'] ?? false) === 'yes');
    }

    public function isNeedReselectAfterChangePS():bool
    {
        return ($this->settings['needReselect'] ?? false) === 'yes';
    }

    public function getOrderMapping():array
    {
        $statusMapping = [];
        foreach (self::getCatapultoStatuses() as $status => $text) {
            $statusMapping[$status] = $this->settings['fp_' . $status] ?? 'not_set';
        }
        return $statusMapping;
    }

}
