<?php

namespace Ipol\Catapulto\Wordpress\Controller;

use Ipol\Catapulto\Api\Entity\EncoderInterface;
use Ipol\Catapulto\Api\Logger\Psr\Log\LoggerInterface;
use Ipol\Catapulto\Catapulto\CatapultoApplication;
use Ipol\Catapulto\Catapulto\Entity\AbstractResult;
use Ipol\Catapulto\Core\Entity\CacheInterface;

abstract class AbstractController
{
    protected $app;
    protected $errors = [];
    protected $response = null;
    protected $markDeliveriedOrderIsPaid = false;
    protected $emailNotifyEnabled = false;
    protected $orderMapping = [];

    public function __construct(
        string $token,
        string $customApiBaseUrl = '',
        int $timeout = 15,
        ?EncoderInterface $encoder = null,
        ?CacheInterface $cache = null,
        ?LoggerInterface $logger = null
    )
    {
        $this->app = new CatapultoApplication($token, $customApiBaseUrl, $timeout, $encoder, $cache, $logger);
    }

    protected function isResponseError(AbstractResult $response):bool
    {
        $this->response = $response;
        if ($response->isSuccess()) return false;

        $errorsCollections = $this->app->getErrorCollection();
        while ($error = $errorsCollections->getNext()) $errors[] = 'R' . $error->getCode() . ': ' . $error->getMessage();

        $errors[] = 'R' . $response->getError()->getCode() . ': ' . $response->getError()->getMessage();
        $this->errors = $errors;
        return true;
    }

    public function getFirstError():string
    {
        if (empty($this->errors)) return '';
        return $this->errors[0];
    }
    public function getErrorsMessageByOneString():string
    {
        $result = '';
        foreach ($this->errors as $err) $result .= $err . PHP_EOL;
        return $result;
    }
    public function getResponse():?AbstractResult
    {
        return $this->response;
    }

    public function setMarkDeliveriedOrderIsPaid(bool $markDeliveriedOrderIsPaid): self
    {
        $this->markDeliveriedOrderIsPaid = $markDeliveriedOrderIsPaid;
        return $this;
    }

    public function setEmailNotifyEnabled(bool $emailNotifyEnabled): self
    {
        $this->emailNotifyEnabled = $emailNotifyEnabled;
        return $this;
    }

    public function setOrderMapping(array $mapping): self
    {
        $this->orderMapping = $mapping;
        return $this;
    }

}
