<?php

namespace Ipol\Catapulto\Wordpress\Controller;

use Ipol\Catapulto\Api\Entity\Response\Shipment;
use Ipol\Catapulto\Core\Delivery\Cargo;
use Ipol\Catapulto\Core\Entity\Money;
use Ipol\Catapulto\Core\Order\Address;
use Ipol\Catapulto\Core\Order\Buyer;
use Ipol\Catapulto\Core\Order\BuyerCollection;
use Ipol\Catapulto\Core\Order\Goods;
use Ipol\Catapulto\Core\Order\Item;
use Ipol\Catapulto\Core\Order\ItemCollection;
use Ipol\Catapulto\Core\Order\Order;
use Ipol\Catapulto\Core\Order\Payment;
use Ipol\Catapulto\Wordpress\CatapultoShippingMethod;
use Ipol\Catapulto\Wordpress\DB\OrdersDB;
use Ipol\Catapulto\Wordpress\Shipping\Calculator;

class OrderController extends AbstractController
{

    public function sendOrder(array $requestData)
    {
        $order = new \WC_Order(intval($requestData['orderId']));
        $rateResultData = $order->get_meta('ctpt_ratedata');
        $rateResultData = json_decode($rateResultData, true);

        $shippingMethod = new CatapultoShippingMethod();
        $moduleSettings = $shippingMethod->getAllSettings();

        $cOrder = new Order();

        $zip = intval($requestData['ctpt_buyerZip']);
        $city = $rateResultData['dadata']['data']['settlement'] ?? '';
        if (empty($city)) $city = $rateResultData['dadata']['data']['city'] ?? '';

        $receiverAddress = (new Address())
            ->setCity($city)
            //->setZip($zip)
            //->setCode() ??
            ->setStreet($requestData['ctpt_buyerStreet'])
            ->setBuilding($requestData['ctpt_buyerBuilding'])
            ->setFlat($requestData['ctpt_buyerDoorNumber'])
            ->setCode($rateResultData['rate_param']['receiver_locality_id'])
        ;
        if (!empty($zip)) $receiverAddress->setZip(intval($zip));
        $orderComment = '';
        if (!empty($requestData['ctpt_comment'])) {
            $receiverAddress->setComment($requestData['ctpt_comment']);
            $orderComment = $requestData['ctpt_comment'];
        }

        $receiverEmail = $order->get_billing_email();
        if (!empty($requestData['ctpt_buyerEmail'])) $receiverEmail = $requestData['ctpt_buyerEmail'];

        $buyer = (new Buyer())
            ->setPatronymic($requestData['ctpt_buyerName'])
            ->setPhone($requestData['ctpt_buyerPhone'])
            ->setField('rateResultId', $rateResultData['variant']['id']);
        if (!empty($requestData['ctpt_buyerCompany'])) $buyer->setField('company', $requestData['ctpt_buyerCompany']);
        $buyerCollection = (new BuyerCollection())->add($buyer);

        $cOrder->setAddressTo($receiverAddress)->setBuyers($buyerCollection);

        $contactResponse = $this->app->contactCreate($cOrder);
        if ($this->isResponseError($contactResponse)) return null;
        $cOrder->getAddressTo()->setCode($contactResponse->getResponse()->getId());

        //Sender
        $senderAddress = (new Address())
            ->setCode($shippingMethod->getSenderId())
            ->setField('senderId', $shippingMethod->getSenderId())
            ->setField('senderCity', $shippingMethod->getSenderCity());

        //Basket
        $itemCollections = new ItemCollection();
        $cargoResult = Calculator::getCurrentCargoDataByOrder($order);
        /** @var Cargo $cargo */
        $cargo = $cargoResult['CARGO'];
        $cargo->reset();
        while ($el = $cargo->getNext()) {
            $item = (new Item())
                ->setName($el->getField('name'))
                ->setQuantity($el->getQuantity())
                ->setPrice($el->getPrice())
                ->setWeight($el->getWeight())
                ->setWidth($el->getWidth())
                ->setLength($el->getLength())
                ->setHeight($el->getHeight())
                ->setArticul($el->getField('sku'))
            ;
            $itemCollections->add($item);
        }

        $goodsAmount = floatval($order->get_total()) - floatval($order->get_shipping_total());
        $goods = new Goods();
        $goods
            ->setWeight($cargoResult['WEI'])
            ->setWidth($cargoResult['W'])
            ->setLength($cargoResult['L'])
            ->setHeight($cargoResult['H'])
        ;

        //Payment
        $payment = (new Payment())->setField('sumToPay', new Money(floatval($requestData['ctpt_sumToPay'])) );

        $pickupDay =  explode('-', (isset($rateResultData['selected']) && isset($rateResultData['selected']['pickup_date'])) ? $rateResultData['selected']['pickup_date'] : $rateResultData['variant']['pickup_day']);
        $pickupDay = implode('.',array_reverse($pickupDay));

        $deliveryDay = explode('-',(isset($rateResultData['selected']) && isset($rateResultData['selected']['delivery_date'])) ? $rateResultData['selected']['delivery_date'] : $rateResultData['variant']['delivery_day']);
        $deliveryDay = implode('.',array_reverse($deliveryDay));

        $needInsurance = false;
        $insuranceValue = 0;
        if ( $requestData['ctpt_needInsurance'] == 'Y' ) {
            $needInsurance = true;
            $insuranceValue = floatval($requestData['ctpt_insuranceValue']);
        }

        $receiverTerminalCode = '';
        if ( isset($rateResultData['Terminal']) && ($rateResultData['ctpt_params']['rate_type'] == 'pvz') ) {
            $receiverTerminalCode = $rateResultData['Terminal']['code'];
        }

        $cOrder
            ->setAddressFrom($senderAddress)
            ->setItems($itemCollections)
            ->setPayment($payment)
            ->setNumber(intval($requestData['orderId']))
            ->setGoods($goods)
            ->setField('rate_id', $rateResultData['variant']['id'])
            ->setField('rateResultId',$rateResultData['variant']['id']) //то же самое значениеюю
            ->setField('pickup_day',$pickupDay)
            ->setField('delivery_day',$deliveryDay)
            ->setField('need_insurance',$needInsurance)
            ->setField('insurance_value',new Money($insuranceValue))
            ->setField('receiver_terminal_code',$receiverTerminalCode)
            ->setField('deliveryPaySide','sender')
            ->setField('additional_services', [])
        ;

        if (
            ($moduleSettings['poaEnabled'] == 'yes')
            && ($rateResultData['variant']['operator'] == 'dellin')
        ) {
            $cOrder
                ->setField('sender_poa', true)
                ->setField('from_name', $moduleSettings['poaFromName'])
                ->setField('from_passport_series', $moduleSettings['poaFromPassportSeries'])
                ->setField('from_passport_number', $moduleSettings['poaFromPassportNumber'])
                ->setField('from_passport_date', $moduleSettings['poaFromPassportDate'])
                ->setField('send_receiver_poa_email', $moduleSettings['poaSendReceiverEmail'])
            ;
        }

        $isPod = $requestData['ctpt_isBeznal'] != 'Y';
        if ($isPod) {
            if ($requestData['ctpt_deliveryPaySide'] == 'Y') {
                $cOrder
                    ->setField('additional_services', ['pod_amount','cod_amount'])
                    ->setField('deliveryPaySide', 'receiver')
                ;
            } else $cOrder->setField('additional_services', ['pod_amount']);
        }

        // for w2* tarifs
        $operatorSendType = $moduleSettings['widgetDeliveryFrom'];
        if (isset($moduleSettings['op_type_' . $rateResultData['variant']['operator'] ])) $operatorSendType = $moduleSettings['op_type_' . $rateResultData['variant']['operator'] ];

        if ($operatorSendType == 'warehouse') $cOrder->setField('sender_terminal_code', $moduleSettings['op_code_' . $rateResultData['variant']['operator'] ] );

        if ($isPod) {
            $shipmentNPResult = $this->app->shipmentNpCreate($cOrder);
            if ($this->isResponseError($shipmentNPResult)) return null;
        }

        $orderResult = $this->app->shipmentCreate($cOrder);
        if ($this->isResponseError($orderResult)) return null;
        /** @var Shipment $response */
        $response = $orderResult->getResponse();

        $orderData = [
            'order_id' => $order->get_id(),
            'number' => $response->getNumber(),
            'tracking_number' => '',
            'tracking_link' => '',
            'key' => $response->getKey(),
            'main_status' => '',
            'main_status_display' => '',
            'weight' => $cargoResult['WEI'],
            'price' => $goodsAmount,
            'operator' => $rateResultData['variant']['operator'],
            'sender_contact_id' => $senderAddress->getCode(),
            'sender_company' => '',
            'receiver_contact_id' => $receiverAddress->getCode(),
            'receiver_locality' => $receiverAddress->getCity(),
            'receiver_name' => $buyer->getPatronymic(),
            'receiver_company' => $buyer->getField('company') ?? '',
            'receiver_phone' => $buyer->getPhone(),
            'receiver_email' => $receiverEmail,
            'receiver_street' => $receiverAddress->getStreet() ?? '',
            'receiver_house' => $receiverAddress->getHouse() ?? '',
            'receiver_flat' => $receiverAddress->getFlat() ?? '',
            'description' => $cargoResult['COMMENT'],
            'with_insurance' => $needInsurance,
            'insurance_cost' => $insuranceValue,
            'documents' => 'a:0:{}',
            'last_tracking_text' => '',
            'problem_text' => '',
            'is_pod' => $isPod,
            'sum_to_pay' => $payment->getField('sumToPay')->getAmount(),
            'payment' => 'a:0:{}', //Пока не знаю для чего нужно это поле
            'tracking_status' => '',
            'rate_result_id' => $rateResultData['variant']['id'],
            'terminal_code' => $receiverTerminalCode,
            'comment' => $orderComment,
            'dlv_pay_side' => ($requestData['ctpt_deliveryPaySide'] == 'Y') ? 1 : 0,
        ];
        $ordersDb = new OrdersDB();
        $ordersDb->saveOrUpdate($orderData, 'order_id');

        return $orderData;
    }

}
