<?php

namespace Ipol\Catapulto\Wordpress\DB;

class AbstractDB
{
    protected $db;
    protected $tableName;
    protected $fullTableName;
    protected static $stringRows = [];

    public function __construct(string $tableName)
    {
        global $wpdb;
        $this->db = $wpdb;
        $this->tableName = $tableName;
        $this->fullTableName = $this->db->get_blog_prefix() . $tableName;
    }

    protected function prepareSQLData(array $data): array
    {
        if (isset($data['id'])) unset($data['id']);
        $data=array_merge($data,['updated'=> (new \DateTime())->format('Y-m-d H:i:s')]);
        $colTypes = [];
        $sData = [];
        foreach ($data as $fname => $value) {
            $type = '%d';
            if (in_array($fname, static::$stringRows)) $type = '%s';
            $colTypes[] = $type;
            $sData[$fname] = $value;
        }
        return [$sData, $colTypes];
    }

    /**
     * @param int $limit
     * @return array 0 - record count, 1 - pages count
     */
    public function getPagesRecordsCount(int $limit): array
    {
        $res = $this->db->get_row("SELECT COUNT(*) as c FROM {$this->fullTableName}", 'ARRAY_A');
        $cnt = intval($res['c']);
        return [$cnt, ceil($cnt / $limit)];
    }
    public function getByID(int $recId): ?array
    {
        $sql = $this->db->prepare("SELECT * FROM {$this->fullTableName} WHERE `id` = %d;", $recId);
        return $this->db->get_row($sql, 'ARRAY_A');
    }

    public function getAll(string $orderBy = 'id', string $orderDirection = 'DESC', int $offset = 0, int $limit = 0): ?array
    {
        $orderDirection = 'DESC' === strtoupper( $orderDirection ) ? 'DESC' : 'ASC';
        $orderBySQL = sanitize_sql_orderby("{$orderBy} {$orderDirection}");
        $sqlQuery = "SELECT * FROM {$this->fullTableName} ORDER BY $orderBySQL ";

        $sql = "";
        if ($limit > 0) {
            $sqlQuery .= " LIMIT %d, %d;";
            $sql = $this->db->prepare($sqlQuery, $offset, $limit);
        } else {
            $sqlQuery .= ';';
            $sql = $this->db->prepare($sqlQuery);
        }
        return $this->db->get_results($sql, 'ARRAY_A');
    }

    protected function insert(array $data): void
    {
        $sqlData = $this->prepareSQLData($data);
        $this->db->insert( $this->fullTableName, $sqlData[0], $sqlData[1] );
    }
    protected function update(array $data, int $recordId): void
    {
        $sqlData = $this->prepareSQLData($data);
        $this->db->update($this->fullTableName, $sqlData[0], ['id'=>$recordId], $sqlData[1], ['%d']);
    }

    /**
     * @param array $data
     * @param string $keyColumn
     * @return string last operation error (or empty string)
     */
    public function saveOrUpdate(array $data, string $keyColumn = ''): string
    {
        if (!empty($keyColumn)) {
            $rec = $this->getByColumn($keyColumn, $data[$keyColumn]);
            if ($rec && isset($rec['id'])) {
                $recId = intval($rec['id']);
                $this->update($data, $recId);
                return $this->db->last_error;
            }
        }
        $this->insert($data);
        return $this->db->last_error;
    }

    public function getByColumn(string $columnName, $value): ?array
    {
        $dType = '%d';
        if (in_array($columnName, static::$stringRows)) $dType = '%s';
        $sql = $this->db->prepare("SELECT * FROM {$this->fullTableName} WHERE `{$columnName}` = {$dType};", $value);
        return $this->db->get_row($sql, 'ARRAY_A');
    }

    public function getAllByColumn(string $columnName, $value): ?array
    {
        $dType = '%d';
        if (in_array($columnName, static::$stringRows)) $dType = '%s';
        $sql = $this->db->prepare("SELECT * FROM {$this->fullTableName} WHERE `{$columnName}` = {$dType};", $value);
        return $this->db->get_results($sql, 'ARRAY_A');
    }

}
