<?php

namespace Ipol\Catapulto\Wordpress\DB;

class CartVariantsDB extends AbstractDB
{
    const TABLENAME = 'catapulto_cartdata';
    protected static $stringRows = [ //varchar, text, datetime
        'variant', 'updated'
    ];

    public function __construct()
    {
        parent::__construct(self::TABLENAME);
    }

    private function updateData(array $data, int $cartId)
    {
        $sqlData = $this->prepareSQLData($data);
        $this->db->update( $this->fullTableName, $sqlData[0], ['wp_cart_id'=>$cartId], $sqlData[1], ['%d']);
    }

    private function updateDataByCtptrate(array $data, int $ctptRate)
    {
        $sqlData = $this->prepareSQLData($data);
        $this->db->update( $this->fullTableName, $sqlData[0], ['ctpt_rate_data'=>$ctptRate], $sqlData[1], ['%d']);
    }

    public function saveCartVariant(int $cartId, array $data)
    {
        $saveData = [
            'wp_cart_id' => $cartId,
            'ctpt_rate_data' => 0,
            'variant' => serialize($data),
        ];
        if (!$this->isCartDataExist($cartId)) $this->insert($saveData);
        else $this->updateData($saveData, $cartId);
    }

    public function deleteCartVariant(int $cartId)
    {
        $sql = $this->db->prepare("DELETE FROM {$this->fullTableName} WHERE `wp_cart_id` = %d;", $cartId);
        $this->db->query($sql);
    }

    public function saveCartVariantByCtptrate(int $ctptRateId, array $data)
    {
        $saveData = [
            'wp_cart_id' => 0,
            'ctpt_rate_data' => $ctptRateId,
            'variant' => serialize($data),
        ];
        if (!$this->isCartDataExistByCtptrate($ctptRateId)) $this->insert($saveData);
        else $this->updateDataByCtptrate($saveData, $ctptRateId);
    }

    private function isCartDataExist(int $cartId):bool
    {
        $sql = $this->db->prepare("SELECT `wp_cart_id` FROM {$this->fullTableName} WHERE `wp_cart_id` = %d;", $cartId);
        $row = $this->db->get_row($sql, 'ARRAY_A');
        return is_array($row) && !empty($row);
    }

    private function isCartDataExistByCtptrate(int $ctptRate):bool
    {
        $sql = $this->db->prepare("SELECT `ctpt_rate_data` FROM {$this->fullTableName} WHERE `ctpt_rate_data` = %d;", $ctptRate);
        $row = $this->db->get_row($sql, 'ARRAY_A');
        return is_array($row) && !empty($row);
    }

    public function getDataByCartID(int $cartId):?array
    {
        $sql = $this->db->prepare("SELECT * FROM {$this->fullTableName} WHERE `wp_cart_id` = %d;", $cartId);
        return $this->db->get_row($sql, 'ARRAY_A');
    }

    public function getDataByCtptrate(int $ctptRateId):?array
    {
        $sql = $this->db->prepare("SELECT * FROM {$this->fullTableName} WHERE `ctpt_rate_data` = %d;", $ctptRateId);
        return $this->db->get_row($sql, 'ARRAY_A');
    }

    public function clearOldVariants()
    {
        $this->db->query("DELETE FROM {$this->fullTableName} WHERE `updated` < DATE_SUB(NOW(), INTERVAL 1 DAY);");
    }


    public function getByID(int $recId): ?array
    {
        return null;
    }

}
