<?php

namespace Ipol\Catapulto\Wordpress\Factory;

use Ipol\Catapulto\Api\Logger\FileRoute;
use Ipol\Catapulto\Api\Logger\Logger;
use Ipol\Catapulto\Wordpress\CatapultoPlugin;
use Ipol\Catapulto\Wordpress\CatapultoShippingMethod;
use Ipol\Catapulto\Wordpress\Controller\AbstractController;

class ControllerFactory
{

    public static function createCatapultoControllerWSettings(
        string $apiKey,
        string $customApiUrl,
        bool $isLogger = false,
        string $controllerName = 'Ipol\Catapulto\Wordpress\Controller\MainController'
    ):AbstractController
    {
        $logger = null;
        if ($isLogger) {
            $route = new FileRoute(CatapultoPlugin::getMainPluginDir() . '/logs/' . (new \DateTime())->format('Y_m_d') . '_api.txt');
            $route->enable();
            $logger = new Logger([$route]);
        }
        return new $controllerName(
            $apiKey,
            $customApiUrl,
            15,
            null,
            null,
            $logger
        );
    }

    public static function createCatapultoController(string $controllerName = 'Ipol\Catapulto\Wordpress\Controller\MainController'):AbstractController
    {
        $logger = null;
        $shippingMethod = new CatapultoShippingMethod();
        if ($shippingMethod->isLogMode()) {
            $route = new FileRoute(CatapultoPlugin::getMainPluginDir() . '/logs/' . (new \DateTime())->format('Y_m_d') . '_api.txt');
            $route->enable();
            $logger = new Logger([$route]);
        }
        $controller = new $controllerName(
            $shippingMethod->getApiKey(),
            $shippingMethod->getCustomApiURL(),
            15,
            null,
            null,
            $logger
        );
        //Set shipping module settings in controller
        $controller
            ->setEmailNotifyEnabled($shippingMethod->isEmailNotidyEnabled())
            ->setOrderMapping($shippingMethod->getOrderMapping())
            ->setMarkDeliveriedOrderIsPaid($shippingMethod->isMarkDeliveriedOrderIsPaid())
        ;
        return $controller;
    }

}
