<?php

namespace Ipol\Catapulto\Wordpress\Frontend;

use Ipol\Catapulto\Wordpress\CatapultoShippingMethod;
use Ipol\Catapulto\Wordpress\DB\OperatorsDB;
use Ipol\Catapulto\Wordpress\DB\OrdersDB;
use Ipol\Catapulto\Wordpress\Shipping\Calculator;
use Ipol\Catapulto\Wordpress\Tools\TemplateRender;
use Ipol\Catapulto\Wordpress\Tools\Tools;

class AdminPages
{
    private static $shippingMethod;
    private static $moduleSettings = [];
    private static $rateResultData = [];

    public static function renderOrderMetaBox()
    {
        self::$shippingMethod = new CatapultoShippingMethod();
        /** @var CatapultoShippingMethod $shippingMethod */
        $shippingMethod = self::$shippingMethod;
        self::$moduleSettings  = $shippingMethod->getAllSettings();
        $order = wc_get_order();

        //getCurrentData
        $catapultoOrderDb = new OrdersDB();
        $catapultoOrder = $catapultoOrderDb->getByColumn('order_id', $order->get_id());

        //currentOrderRateData
        self::$rateResultData = json_decode($order->get_meta('ctpt_ratedata'), true);
        if (!self::$rateResultData) self::$rateResultData = [];

        //if sended....
        if (!empty($catapultoOrder)) self::renderOrderSendedMetabox($order, $catapultoOrder);
        else self::renderOrderSendMetabox($order);
    }

    private static function renderOrderSendMetabox(\WC_Order $order)
    {
        $orderDefaultGabs = Calculator::getCurrentCargoDataByOrder($order);

        $orderData = $order->get_data();
        $paymentnethod = $orderData['payment_method'];
        if (!is_array(self::$moduleSettings['payCard'])) self::$moduleSettings['payCard'] = [];
        if (!is_array(self::$moduleSettings['payNal'])) self::$moduleSettings['payNal'] = [];
        $isCard = in_array($paymentnethod, self::$moduleSettings['payCard']);
        $isCash = in_array($paymentnethod, self::$moduleSettings['payNal']);
        $serviceFilter = '';
        $isCod = false;
        $codAmount = 0;
        if ($isCash || $isCard) {
            $isCod = true;
            $serviceFilter = 'NP,COD';
            foreach (self::$rateResultData['variant']['additional_services'] as $service) {
                if ($service['name'] == 'cod_amount') {
                    $codAmount = $service['cost'];
                    break;
                }
            }
        }

        $defaultDeliveryFrom = self::$moduleSettings['widgetDeliveryFrom'];
        $inverceDeliveryTypes = [];
        $operatorsDB = new OperatorsDB();
        $allOperators = $operatorsDB->getAllByColumn('op_enabled','Y');
        foreach ($allOperators as $operator) {
            if (self::$moduleSettings['op_type_' . $operator['operator_id']] != $defaultDeliveryFrom) $inverceDeliveryTypes[] = $operator['operator_id'];
        }
        $inverceDeliveryTypesStr = implode("','",$inverceDeliveryTypes);
        if (empty($inverceDeliveryTypesStr)) $inverceDeliveryTypesStr = '[]';
        else $inverceDeliveryTypesStr = "['" . $inverceDeliveryTypesStr . "']";

        //get Order Meta Data
        $receiverPostalCode = self::$rateResultData['dadata']['data']['postal_code'] ?? '';
        $receiverCity = self::$rateResultData['dadata']['data']['settlement'] ?? '';
        if (empty($receiverCity)) $receiverCity = self::$rateResultData['dadata']['data']['city'] ?? '';

        $receiverStreet = self::$rateResultData['dadata']['data']['street'] ?? '';
        $receiverHouse = self::$rateResultData['dadata']['data']['house'] ?? '';
        $receiverFlat = self::$rateResultData['dadata']['data']['flat'] ?? '';

        $pvzSender = '';
        if ( (self::$moduleSettings[ 'op_type_' . self::$rateResultData['variant']['operator'] ] ?? '') == 'warehouse' ) $pvzSender = '<p><b>Код ПВЗ отправителя</b></p><p>' . self::$moduleSettings[ 'op_code_' . self::$rateResultData['variant']['operator'] ] . '</p>';
        $pvzReceiver = '';
        if (isset(self::$rateResultData['Terminal']) && !empty(self::$rateResultData['Terminal'])) {
            $pvzReceiver = '<p><b>Код ПВЗ получателя</b></p>' . (self::$rateResultData['Terminal']['code'] ?? '') . ' - '. self::$rateResultData['Terminal']['address'] . '</p>';
            //Get dadata address for this PVZ
            $city = self::$rateResultData['dadata']['data']['settlement_with_type'];
            if (empty($city)) $city = self::$rateResultData['dadata']['data']['city_with_type'];
            $query = $city . ' ' . self::$rateResultData['Terminal']['address'];
            $resp = wp_remote_post('https://suggestions.dadata.ru/suggestions/api/4_1/rs/suggest/address',[
                'headers' => [
                    'Authorization'=>'Token '.self::$moduleSettings['dadataApikey'],
                    'Content-Type'=>'application/json',
                ],
                'body' => json_encode([
                    'query'=>$query,
                    'count'=>1,
                ]),
            ]);

            if (is_array($resp) && isset($resp['body'])) {
                $result = json_decode($resp['body'], true);
                if (
                    is_array($result)
                    && isset($result['suggestions'])
                    && is_array($result['suggestions'])
                    && (count($result['suggestions']) > 0)
                ) {
                    $receiverStreet = $result['suggestions'][0]['data']['street'] ?? '';
                    $receiverHouse = $result['suggestions'][0]['data']['house'] ?? '';
                    $receiverFlat = $result['suggestions'][0]['data']['flat'] ?? '';
                    if (!empty($result['suggestions'][0]['data']['postal_code'])) $receiverPostalCode = $result['suggestions'][0]['data']['postal_code'];
                }
            }
        }

        $goodsAmount = floatval($order->get_total()) - floatval($order->get_shipping_total());

        $widgetDeliveryTypes = "'".self::$moduleSettings['widgetDeliveryTypes']."'";
        if ($widgetDeliveryTypes == "'All'") $widgetDeliveryTypes = 'false';

        $selectedPickupDate = '';
        $selectedDeliveryDate = '';
        if (isset(self::$rateResultData['selected']) && is_array(self::$rateResultData['selected']) && count(self::$rateResultData['selected']) > 0) {
            $selectedPickupDate = '<p><b>Выбранная дата забора</b></p><p>'.self::$rateResultData['selected']['pickup_date'].'</p>';
            $selectedDeliveryDate = '<p><b>Выбранная дата доставки</b></p><p>'.self::$rateResultData['selected']['delivery_date'].'</p>';
        }

        $emptyRateData = [
            'dadata' => [
                'data' => [],
            ],
            'variant' => [
                'additional_services' => [],
                'operator' => 'NotSet',
            ],
            'ctpt_params' => [
                'isEmpty' => true,
            ],
        ];
        $str = json_encode($emptyRateData);

        $orderData = [
            'nonce' => wp_create_nonce('wp_rest'),
            'isEmpty' => (self::$rateResultData['ctpt_params']['isEmpty'] ?? true) ? 'true' : 'false',
            'id'         => $order->get_id(),
            'first_name' => $order->get_shipping_first_name() ? : $order->get_billing_first_name(),
            'last_name'  => $order->get_shipping_last_name() ? : $order->get_billing_last_name(),
            'phone'      => Tools::phoneValidator($order->get_billing_phone()),
            'email'      => $order->get_billing_email(),
            'wei' => $orderDefaultGabs['WEI'],
            'wid' => $orderDefaultGabs['W'],
            'len' => $orderDefaultGabs['L'],
            'hei' => $orderDefaultGabs['H'],
            'cargoComment' => $orderDefaultGabs['COMMENT'],
            'receiverZip' => $receiverPostalCode,
            'receiverCity' => $receiverCity,
            'receiverStreet' => $receiverStreet,
            'receiverHouse' => $receiverHouse,
            'receiverFlat' => $receiverFlat,

            'senderId' => self::$moduleSettings['senderId'],
            'senderLocId' => self::$moduleSettings['senderLocalityId'],
            'senderZip' => self::$moduleSettings['senderZip'],
            'senderCity' => self::$moduleSettings['senderCity'],

            'variantId' => self::$rateResultData['variant']['id'] ?? '',
            'operator' => self::$rateResultData['variant']['operator'] ?? '',
            'pvzSender' => $pvzSender,
            'pvzReceiver' => $pvzReceiver,
            'pickupDate' => self::$rateResultData['variant']['pickup_day'] ?? '',
            'deliveryDate' => self::$rateResultData['variant']['delivery_day'] ?? '',
            'operatorTarif' => self::$rateResultData['variant']['rate'] ?? '',

            'deliveryPrice' => self::$rateResultData['variant']['price'] ?? '0',
            'goodsPrice' => $goodsAmount,
            'shippingPrice' => floatval($order->get_shipping_total()),
            'isCod' => $isCod ? 'true' : 'false',
            'codAmount' => $codAmount,

            'dadataToken' => self::$moduleSettings['dadataApikey'],

            'widgetDeliveryFrom' => self::$moduleSettings['widgetDeliveryFrom'],
            'widgetMapMode' =>  (self::$moduleSettings['enMapOpenMode'] === 'courier') ? 'false' : 'true',
            'inverseOperators' => $inverceDeliveryTypesStr,
            'isCard' => $isCard ? 'true' : 'false',
            'isCash' => $isCash ? 'true' : 'false',
            'needInsurance' => (self::$moduleSettings['mindEnsurance'] === 'yes') ? 'true' : 'false',
            'day_shift' => intval(self::$moduleSettings['termIncrease']),
            'widgetDeliveryTypes' => $widgetDeliveryTypes,
            'serviceFilter' => $serviceFilter,

            'selectedPickupDate' => $selectedPickupDate,
            'selectedDeliveryDate' => $selectedDeliveryDate,

            'isInsurance' => (self::$rateResultData['ctpt_params']['insurance'] ?? false) ? 'true' : 'false',

        ];

        $templateRender = new TemplateRender();
        $templateRender->render('admin/orderPanel.tpl', $orderData, true);
        Tools::baseCssLoad('send_panel');
        Tools::baseScriptLoad('send_panel');
    }

    private static function renderOrderSendedMetabox(\WC_Order $order, $catapultoOrder)
    {
        $isCancelled = $catapultoOrder['main_status'] === 'rejected';

        $orderData = [
            'nonce' => wp_create_nonce('wp_rest'),
            'id'    => $order->get_id(),
            'orderStatus' => $isCancelled ? '<span style="color: #f00">'.$catapultoOrder['main_status_display'].'</span>' : $catapultoOrder['main_status_display'],
            'catapultoOrderNum' => $catapultoOrder['number'],
            'catapultoOrderKey' => $catapultoOrder['key'],
            'ctpt_cancelled' => $isCancelled ? 'true' : 'false',
            'problemText' => (empty($catapultoOrder['problem_text'])) ? '' : '<p style="color:#f00">'.$catapultoOrder['problem_text'].'</p>',
            'senderId' => self::$moduleSettings['senderId'],
            'senderCity' => self::$moduleSettings['senderCity'],
            'receiver_name' => $catapultoOrder['receiver_name'],
            'receiver_phone' => $catapultoOrder['receiver_phone'],
            'receiver_email' => $catapultoOrder['receiver_email'],
            'receiver_company' => $catapultoOrder['receiver_company'],
            'receiverZip' => self::$rateResultData['dadata']['data']['postal_code'] ?? '',
            'receiverCity' => $catapultoOrder['receiver_locality'],
            'receiverStreet' => $catapultoOrder['receiver_street'],
            'receiverHouse' => $catapultoOrder['receiver_house'],
            'receiverFlat' => $catapultoOrder['receiver_flat'],
            'variantId' => $catapultoOrder['rate_result_id'],
            'operator' => $catapultoOrder['operator'],
            'pvzSender' => '',
            'pvzReceiver' => '',
            'pickupDate' => $catapultoOrder['pickup_day'],
            'deliveryDate' => $catapultoOrder['delivery_day'],
            'operatorTarif' => self::$rateResultData['variant']['rate'],
            'ctpt_comment' => $catapultoOrder['comment'] ?? '',
            'shippingPrice' => self::$rateResultData['variant']['price'],
            'needInsurance' => intval($catapultoOrder['with_insurance']) > 0 ? 'Да' : 'Нет',
            'insuranceVal' => floatval($catapultoOrder['insurance_cost']),
            'orderPaid' => (intval($catapultoOrder['is_pod']) == 1) ? 'Нет' : 'Да',
            'sumToPay' => floatval($catapultoOrder['sum_to_pay']),
            'deliveryPaySide' => (intval($catapultoOrder['dlv_pay_side']) == 1) ? 'Получатель' : 'Отправитель',

        ];

        $templateRender = new TemplateRender();
        $templateRender->render('admin/orderPanelSended.tpl', $orderData, true);
        Tools::baseCssLoad('send_panel');
    }


}
