<?php

namespace Ipol\Catapulto\Wordpress\Frontend;

use Automattic\WooCommerce\Blocks\Integrations\IntegrationInterface;
use Ipol\Catapulto\Wordpress\CatapultoPlugin;
use Ipol\Catapulto\Wordpress\CatapultoShippingMethod;
use Ipol\Catapulto\Wordpress\Controller\MainController;
use Ipol\Catapulto\Wordpress\DB\CartVariantsDB;
use Ipol\Catapulto\Wordpress\Shipping\Calculator;
use Ipol\Catapulto\Wordpress\Tools\Tools;

class CheckoutBlock implements IntegrationInterface
{
    public function get_name()
    {
        return CatapultoPlugin::PLUGIN_ID;
    }

    public function initialize()
    {
        Tools::blocksScriptLoad('catapulto-checkout-block-frontend');
        Tools::blocksScriptLoad('catapulto-checkout-block-editor');
        Tools::baseScriptLoad('jquery.suggestions');
    }

    public function get_script_handles()
    {
        return ['catapulto-checkout-block-frontend'];
    }

    public function get_editor_script_handles()
    {
        return ['catapulto-checkout-block-editor'];
    }

    public function get_script_data()
    {
        $shippingMethod = new CatapultoShippingMethod();
        $settings = $shippingMethod->getAllSettings();
        $cargo = Calculator::getCurrentCargoDataByCart();
        if (empty($cargo)) return [];

        //get cart && delete current variant id
        $cartVariants = new CartVariantsDB();
        if (WC()->cart && WC()->session) { //current order id in session
            $cartVariants->deleteCartVariant(intval(WC()->session->get('store_api_draft_order') ?? 0));
        }

        $mainController = new MainController(
            $shippingMethod->getApiKey(),
            $shippingMethod->getCustomApiURL(),
            15,
            null,
            null,
            null
        );

        $widgetDeliveryFrom = $settings['widgetDeliveryFrom'];
        $operatorsSettings = [];
        $inverseOperators = [];
        $freeDeliveryOperators = [];
        $icons = [];
        $companyIcons = $mainController->getCompanyIcons();
        if ($companyIcons) {
            $elements = $companyIcons->getCompanies();
            while ($el = $elements->getNext()) {
                $icons[$el->getOperatorId()] = $el->getIcon();

                $operatorSetting = [
                    'operator' => $el->getOperatorId(),
                    'inverse_delivery_type' => false,
                    'free_delivery' => false,
                ];

                if ($settings['op_type_' . $el->getOperatorId()] != $widgetDeliveryFrom ) {
                    $inverseOperators[] = $el->getOperatorId();
                    $operatorSetting['inverse_delivery_type'] = true;
                }

                if ($settings['op_free_' . $el->getOperatorId()] === 'yes' ) {
                    $freeDeliveryOperators[] = $el->getOperatorId();
                    $operatorSetting['free_delivery'] = true;
                }
                $operatorsSettings[] = $operatorSetting;
            }
        }

        $widgetDeliveryTypes = $settings['widgetDeliveryTypes'];
        if ($widgetDeliveryTypes == 'All') $widgetDeliveryTypes = false;

        $params = [
            'isTestMode' => $settings['isTest'] === 'yes',
            'servicePath' => '/?rest_route=/catapultodelivery/widgethandler',
            'senderLocalityId' => $settings['senderLocalityId'],
            'senderZip' => $settings['senderZip'],
            'senderCityFrom' => $settings['senderCity'],
            'dadataToken' => $settings['dadataApikey'],
            'cargoHeight' => $cargo['H'],
            'cargoLength' => $cargo['L'],
            'cargoWidth' => $cargo['W'],
            'cargoWeight' => $cargo['WEI'],
            'cargoComment' => $cargo['COMMENT'],
            'insuredValue' => $cargo['CARGO']->getTotalPrice()->getAmount(),
            'needInsurance' => $settings['mindEnsurance'] === 'yes',

            'PSCard' => $settings['payCard'],
            'PSCash' => $settings['payNal'],

            'widgetDeliveryTypes' => $widgetDeliveryTypes,
            'deliveryType' => $settings['widgetDeliveryFrom'],
            'inverseOperators' => $inverseOperators,
            'freeDeliveryOperators' => $freeDeliveryOperators,
            'operatorsSettings' => $operatorsSettings,
            'dayShift' => intval($settings['termIncrease']),
            'mapOpenMode' => $settings['enMapOpenMode'] === 'map',
            'runOnStart' => $settings['runWidgetOnStart'] === 'yes',
            'customWss' => ($settings['isTest'] === 'yes') ? $settings['customWSUrl'] : '',

            'enDadataSG' => $settings['enDadataSuggestions'] === 'yes',

            'opIcons' => $icons,

            //
            'geoEmptyMessage' => $settings['geoEmptyMessage'] ?? '',
            'freeDlvMinCourier' => floatval($settings['freeDlvMinCourier']) ?? 0,
            'freeDlvMinPVZ' => floatval($settings['freeDlvMinPVZ']) ?? 0,

        ];

        return $params;
    }
}
