<?php

namespace Ipol\Catapulto\Wordpress\Rest;

use Ipol\Catapulto\Core\Order\Address;
use Ipol\Catapulto\Core\Order\Item;
use Ipol\Catapulto\Core\Order\ItemCollection;
use Ipol\Catapulto\Core\Order\Order;
use Ipol\Catapulto\Wordpress\CatapultoPlugin;
use Ipol\Catapulto\Wordpress\CatapultoShippingMethod;
use Ipol\Catapulto\Wordpress\Controller\MainController;
use Ipol\Catapulto\Wordpress\DB\CartVariantsDB;
use Ipol\Catapulto\Wordpress\DB\OperatorsDB;
use Ipol\Catapulto\Wordpress\DB\OrdersDB;
use Ipol\Catapulto\Wordpress\Factory\ControllerFactory;
use Ipol\Catapulto\Wordpress\Tools\Variables;

class RestHandler
{

    public static function widgetHandler($request)
    {
        $requestData = $request->get_params();

        $method = $requestData['METHOD'];
        $params = $requestData['PARAMS'];

        $shippingMethod = new CatapultoShippingMethod();
        $mainController = ControllerFactory::createCatapultoControllerWSettings(
            $shippingMethod->getApiKey(),
            $shippingMethod->getCustomApiURL(),
            $shippingMethod->isLogMode()
        );

        switch ($method) {
            case 'get_ws':
                $result = $mainController->getWSToken();
                if ($result === false) return new \WP_REST_Response(['error'=>$mainController->getFirstError()], 200);
                return new \WP_REST_Response(['ws_token'=> $result->getWsToken()], 200);
            case 'create_rate':
                $requestData = json_decode($params, true);
                if (empty($requestData)) return new \WP_REST_Response(['error'=>'CreateRate: DATA_CORRUPTED'], 200);

                $ignoredSettlementTypes = Variables::getIgnoredSettlementTypes();
                $receiverCity = $requestData['dadata_selected_choice']['settlement'] ?? '';
                $zip = $requestData['dadata_selected_choice']['postal_code'] ?? '';

                $fiasLevel = intval($requestData['dadata_selected_choice']['fias_level']);
                $cityFiasId = $requestData['dadata_selected_choice']['city_fias_id'];
                $settlementType = $requestData['dadata_selected_choice']['settlement_type'];
                $settlementFiasId = $requestData['dadata_selected_choice']['settlement_fias_id'];
                $countryIso = $requestData['dadata_selected_choice']['country_iso_code'] ?? 'ru';
                $countryIso = mb_strtolower($countryIso);

                if (
                    empty($receiverCity)
                    || ($fiasLevel > 64)
                    || (in_array($requestData['dadata_selected_choice']['settlement_type'], $ignoredSettlementTypes))
                ) $receiverCity = $requestData['dadata_selected_choice']['city'] ?? '';

                if (empty($receiverCity)) {
                    return new \WP_REST_Response(['error'=>'GEO_DATA_EMPTY'], 200);
                }

                //receiverAddress
                $geoData = $mainController->getGeo($zip, $receiverCity, $countryIso, 1, $cityFiasId, $settlementFiasId, $fiasLevel, $settlementType);
                if (!$geoData) return new \WP_REST_Response(['error'=>$mainController->getFirstError()], 200);
                if ($geoData->getQuantity() === 0) return new \WP_REST_Response(['error'=>'Geo: location not found'], 200);

                $receiver = (new Address())
                    ->setZip($geoData->getFirst()->getZip())
                    ->setField('cityFrom', $requestData['location']['term'])
                    ->setField('locality_id', $geoData->getFirst()->getId());

                $sender = (new Address())
                    ->setZip($shippingMethod->getSenderZip())
                    ->setField('cityFrom', $shippingMethod->getSenderCity())
                    ->setField('locality_id', $shippingMethod->getSenderLocId());

                $itemCollection = new ItemCollection();
                if (!empty($requestData['cargo_data'])) {
                    $item = (new Item())
                        ->setField('comment', $requestData['cargo_data']['cargo_comment'] ?? 'empty')
                        ->setField('type', $requestData['delivery_type'] ?? 'parcel');
                    ;
                    if (isset($requestData['cargo_data']['width']) && !empty($requestData['cargo_data']['width'])) $item->setWidth( intval($requestData['cargo_data']['width']) );
                    if (isset($requestData['cargo_data']['height']) && !empty($requestData['cargo_data']['height'])) $item->setHeight( intval($requestData['cargo_data']['height']) );
                    if (isset($requestData['cargo_data']['length']) && !empty($requestData['cargo_data']['length'])) $item->setLength( intval($requestData['cargo_data']['length']) );
                    if (isset($requestData['cargo_data']['weight']) && !empty($requestData['cargo_data']['weight'])) $item->setWeight( intval($requestData['cargo_data']['weight']) );
                    if (isset($requestData['cargo_data']['quantity']) && !empty($requestData['cargo_data']['quantity'])) $item->setQuantity( intval($requestData['cargo_data']['quantity']) );
                    $itemCollection->add($item);
                }

                $order = (new Order())
                    ->setAddressFrom($sender)
                    ->setAddressTo($receiver)
                    ->setItems($itemCollection);

                $cargoData = $mainController->createCargo($order);
                if (!$cargoData) return new \WP_REST_Response(['error'=>$mainController->getFirstError()], 200);
                if (empty($cargoData->getId())) return new \WP_REST_Response(['error'=>'Cargo: empty data'], 200);
                $order->setField('cargoes',[$cargoData->getId()]);

                $iconData = [];
                $icons = $mainController->getCompanyIcons();
                if (!$icons) return new \WP_REST_Response(['error'=>$mainController->getFirstError()], 200);

                while ($icon = $icons->getCompanies()->getNext()) {
                    $iconData[$icon->getOperatorId()] = $icon->getFields();
                }

                $order->setField('dadata_variant', $requestData['dadata_selected_choice']);
                if ($shippingMethod->getSenderId() > 0) $order->setField('sender_contact_id', $shippingMethod->getSenderId());
                if (isset($requestData['pickup_days_shift'])) $order->setField('pickup_days_shift', $requestData['pickup_days_shift']);

                $rate = $mainController->createRate($order);
                if (!$rate) return new \WP_REST_Response(['error'=>$mainController->getFirstError()], 200);
                if (empty($rate->getKey())) return new \WP_REST_Response(['error'=>'CANNOT_CREATE_RATES'], 200);

                return new \WP_REST_Response([
                    'key' => $rate->getKey(),
                    'locations' => [
                        'sender'  => [
                            'locality_id' => $shippingMethod->getSenderLocId(),
                            'zip'         => $shippingMethod->getSenderZip(),
                        ],
                        'contact' => [
                            'locality_id' => $order->getAddressTo()->getField('locality_id'),
                            'zip'         => $order->getAddressTo()->getZip()
                        ],
                    ],
                    'params'    => [
                        'sender_locality_id'   => $shippingMethod->getSenderLocId(),
                        'receiver_locality_id' => $order->getAddressTo()->getField('locality_id'),
                        'cargoes'             => [$cargoData->getId()]
                    ],
                    'icons' => $iconData
                ],200);

            case 'get_rate':
                $requestData = json_decode($params, true);
                if (empty($requestData)) return new \WP_REST_Response(['error'=>'GetRate: DATA_CORRUPTED'], 200);

                if (empty($requestData['rate_id']) || !is_string($requestData['rate_id'])) return new \WP_REST_Response(['error'=>'GetRate: RATE_ID_IS_EMPTY'], 200);
                $rateId = $requestData['rate_id'];

                $filter = [
                    'shipping_type_filter' => $requestData['shipping_type_filter'] ? : 'd2d',
                    'pickup_days_shift'    => 0
                ];
                if ($requestData['pickup_days_shift'] > 0 && $requestData['pickup_days_shift'] <= 365) $filter['pickup_days_shift'] = intval($requestData['pickup_days_shift']);

                if ($requestData['services_filter'] && in_array($requestData['services_filter'], ['NP', 'COD', 'NP,COD'])) $filter['services_filter'] = $requestData['services_filter'];

                $rates = $mainController->getRate($rateId, $filter, (isset($requestData['need_insurance']) ? ($requestData['need_insurance'] === true) : false), $requestData['insured_value'] ?? 0);
                if (!$rates) return new \WP_REST_Response(['error'=>$mainController->getFirstError()], 200);

                $result = [
                    'count'          => $rates->getCount(),
                    'rate_completed' => $rates->isRateCompleted(),
                    'results'        => []
                ];

                while ($item = $rates->getResults()->getNext()) {
                    $result['results'][] = $item->getAllFields();
                }

                return new \WP_REST_Response($result, 200);

            case 'get_tariff':
                $requestData = json_decode($params, true);
                if (empty($requestData)) return new \WP_REST_Response(['error'=>'GetTarif: DATA_CORRUPTED'], 200);
                if (empty($requestData['tariff_id'])) return new \WP_REST_Response(['error'=>'GetTarif: TARIFF_ID_IS_EMPTY'], 200);

                $daysShift = 0;

                if ($requestData['pickup_days_shift'] > 0 && $requestData['pickup_days_shift'] <= 365) $daysShift = intval($requestData['pickup_days_shift']);

                $tarif = $mainController->getTarif($requestData['tariff_id'], $daysShift);
                if (!$tarif) return new \WP_REST_Response(['error'=>$mainController->getFirstError()], 200);
                return new \WP_REST_Response([
                    [
                        'id'         => $tarif->getId(),
                        'time-slots' => $tarif->getTimeSlots()
                    ]
                ], 200);

            case 'get_terminals':
                $requestData = json_decode($params, true);
                if (empty($requestData)) return new \WP_REST_Response(['error'=>'GetTerminals: DATA_CORRUPTED'], 200);

                if (empty($requestData['terminal_request_data']['sender_locality_id']) || empty($requestData['terminal_request_data']['receiver_locality_id'])) return new \WP_REST_Response(['error'=>'GetTerminals: EMPTY_TERMINAL_DATA'], 200);

                $addressFrom = (new Address())->setField('locality_id',$requestData['terminal_request_data']['sender_locality_id']);
                $addressTo = (new Address())->setField('locality_id',$requestData['terminal_request_data']['receiver_locality_id']);

                $page = 1;
                if (isset($requestData['page'])) $page = intval($requestData['page']);

                $order = (new Order())
                    ->setAddressFrom($addressFrom)
                    ->setAddressTo($addressTo)
                    ->setField('limit', 300)
                    ->setField('page', $page )
                ;

                if ($requestData['terminal_request_data']['company']) $order->setField('company', $requestData['terminal_request_data']['company']);
                if (isset($requestData['services_filter']) && !empty($requestData['services_filter'])) $order->setField('services_filter', $requestData['services_filter']);
                if (isset($requestData['terminal_request_data']['cargoes']) && is_array($requestData['terminal_request_data']['cargoes']) && !empty($requestData['terminal_request_data']['cargoes'])) $order->setField('cargoes', $requestData['terminal_request_data']['cargoes']);

                if (isset($requestData['terminal_request_data']['lat']) && floatval($requestData['terminal_request_data']['lat']) > 0)
                    $order->setField('lat', floatval($requestData['terminal_request_data']['lat']));

                if (isset($requestData['terminal_request_data']['lon']) && floatval($requestData['terminal_request_data']['lon']) > 0)
                    $order->setField('lon', floatval($requestData['terminal_request_data']['lon']));

                if (isset($requestData['terminal_request_data']['radius_km']) && floatval($requestData['terminal_request_data']['radius_km']) > 0)
                    $order->setField('radius_km', floatval($requestData['terminal_request_data']['radius_km']));

                $terminals = $mainController->getTerminals($order);
                if (!$terminals) return new \WP_REST_Response(['error'=>$mainController->getFirstError()], 200);
                if ($terminals->getStatus() !== 'ok') return new \WP_REST_Response(['error'=>'GetTerminals: Status not OK'], 200);

                $result = [
                    'data'   => $terminals->getData()->getAllFields(),
                    'status' => $terminals->getStatus()
                ];
                foreach ($result['data']['data'] as &$point) {
                    $point['point-type']=$point['point_type'];
                    unset($point['point_type']);
                }
                unset($point);
                return new \WP_REST_Response($result, 200);

            case 'get_terminal':
                $requestData = json_decode($params, true);
                if (empty($requestData)) return new \WP_REST_Response(['error'=>'GetOneTerminal: DATA_CORRUPTED'], 200);
                if (empty($requestData['terminal_id'])) return new \WP_REST_Response(['error'=>'GetOneTerminal: TERMINAL_ID_IS_EMPTY'], 200);

                $terminal = $mainController->getTerminal(intval($requestData['terminal_id']));
                if (!$terminal) return new \WP_REST_Response(['error'=>$mainController->getFirstError()], 200);

                $result = [
                    'status' => $terminal->getStatus() ?? null,
                    'data'   => []
                ];

                while ($item = $terminal->getData()->getNext()) {
                    $point = $item->getAllFields();
                    $point['point-type'] = $point['point_type'];
                    unset($point['point_type']);
                    $result['data'][] = $point;
                }
                return new \WP_REST_Response($result, 200);

        }

        return new \WP_REST_Response(['error'=>'Unknown error'], 200);
    }

    public static function syncOperators($request)
    {
        $requestData = $request->get_params();

        $operatorsDB = new OperatorsDB();
        $mainController = ControllerFactory::createCatapultoController();
        $icons = $mainController->getCompanyIcons();
        if (!$icons) return new \WP_REST_Response(['error'=>$mainController->getFirstError()], 200);
        $operators = $icons->getCompanies();
        $operatorsDB->disableAllOperators();
        while ($op = $operators->getNext()) {
            $upd = (new \DateTime())->format('Y-m-d H:i:s');
            $operatorData = [
                'operator_id' => $op->getOperatorId(),
                'operator_display' => $op->getOperatorDisplay(),
                'op_enabled' => 'Y',
                'icon' => $op->getIcon(),
                'small_icon' => $op->getSmallIcon(),
                'png_icon' => $op->getPngIcon(),
                'sync_last_date' => $upd,
                'sync_hash' => '',
            ];
            $operatorsDB->saveOrUpdate($operatorData, 'operator_id');
        }

        return new \WP_REST_Response(['res'=>true], 200);
    }

    public static function orderReselectWidgetRate($request)
    {
        $requestData = $request->get_params();

        $rateDataString = $requestData['data'];
        $rateData = json_decode($rateDataString, true);
        $orderId = intval($rateData['ctpt_params']['order_id']);
        $order = new \WC_Order($orderId);
        $order->update_meta_data('ctpt_ratedata', $rateDataString);

        foreach ($order->get_items('shipping') as $shipping_item) {
            /**
             * @var \WC_Order_Item_Shipping $shipping_item
             */
            $order->remove_item($shipping_item->get_id());
        }

        $rate = new \WC_Shipping_Rate();
        $rate->set_id(CatapultoPlugin::PLUGIN_ID . ':pickup');
        $rate->set_cost($rateData['variant']['price']);

        $new_shipping_item = new \WC_Order_Item_Shipping;
        $new_shipping_item->set_shipping_rate($rate);
        $new_shipping_item->set_method_id(CatapultoPlugin::PLUGIN_ID);
        $order->add_item($new_shipping_item);
        $order->calculate_totals();

        $order->save();

        return new \WP_REST_Response(['res'=>true], 200);
    }

    //OrderAdminPanel
    public static function sendOrder($request)
    {
        $requestData = $request->get_params();

        $orderController = ControllerFactory::createCatapultoController('Ipol\Catapulto\Wordpress\Controller\OrderController');
        $createdOrder = $orderController->sendOrder($requestData);
        if ($createdOrder === null) return new \WP_REST_Response(['res'=>false, 'error'=>$orderController->getFirstError()], 200);

        $mainController = ControllerFactory::createCatapultoController();
        $orderInfo = $mainController->getOrderInfo($createdOrder['key']);
        if (!$orderInfo) return new \WP_REST_Response(['res'=>false, 'error'=>$mainController->getFirstError()], 200);

        return new \WP_REST_Response(['res'=>true], 200);
    }

    public static function renewOrder($request)
    {
        $requestData = $request->get_params();

        $mainController = ControllerFactory::createCatapultoController();
        $orderInfo = $mainController->getOrderInfo($requestData['catapultoOrderKey']);
        if (!$orderInfo) return new \WP_REST_Response(['res'=>false, 'error'=>$mainController->getFirstError()], 200);

        return new \WP_REST_Response(['res'=>true], 200);
    }

    public static function cancelOrder($request)
    {
        $requestData = $request->get_params();

        $mainController = ControllerFactory::createCatapultoController();
        $newStatus = $mainController->cancelOrder($requestData['catapultoOrderKey']);
        if (!$newStatus) {
            $errorMsg = $mainController->getFirstError();
            if (empty($errorMsg)) $errorMsg = 'Неизвестная ошибка отмены отправления';
            return new \WP_REST_Response(['res'=>false, 'error'=>$errorMsg], 200);
        }

        //updateOrderInfo
        $mainController->getOrderInfo($requestData['catapultoOrderKey']);

        return new \WP_REST_Response(['res'=>true], 200);
    }


    //Mass Functions in CatapultoOrderPage
    public static function massRenewStatuses($request)
    {
        $requestData = $request->get_params();
        $ids = json_decode($requestData['ids'], true);
        $errors = [];

        /** @var MainController $mainController */
        $mainController = ControllerFactory::createCatapultoController();
        foreach ($ids as $id) {
            $result = $mainController->getOrderInfo($id['k']);
            if (!$result) $errors[] = 'Ошибка обновления заказа ' . $id['w'];
        }

        return new \WP_REST_Response(['res'=>true, 'errors' => $errors], 200);
    }

    public static function massCancelOrders($request)
    {
        $requestData = $request->get_params();
        $ids = json_decode($requestData['ids'], true);
        $errors = [];

        /** @var MainController $mainController */
        $mainController = ControllerFactory::createCatapultoController();
        foreach ($ids as $id) {
            $result = $mainController->cancelOrder($id['k']);
            if (!$result) $errors[] = 'Ошибка отмены заказа ' . $id['w'];
            else $mainController->getOrderInfo($id['k']);
        }

        return new \WP_REST_Response(['res'=>true, 'errors' => $errors], 200);
    }

    public static function massGetPrintDocs($request)
    {
        $requestData = $request->get_params();
        $ids = json_decode($requestData['ids'], true);
        $docType = $requestData['docType'];

        $ordersDB = new OrdersDB();
        $orderIds = [];
        foreach ($ids as $id) $orderIds[] = $id['i'];
        $orders = $ordersDB->getOrdersByIDS($orderIds);

        $result = [];
        foreach ($orders as $order) {
            $orderDocs = unserialize($order['documents']);
            if (!$orderDocs) $orderDocs = [];
            $resultDocs = [];
            foreach ($orderDocs as $doc) {
                if ($doc['title'] === CatapultoPlugin::POA_DOCUMENT_TITLE) $doc['key'] = 'POA';
                if (empty($doc['url'])) continue;
                if ($doc['key'] == $docType) $resultDocs[] = $doc['url'];
            }
            $result[] = [
                'id' => $order['id'],
                'docs' => $resultDocs,
            ];
        }

        return new \WP_REST_Response(['res'=>true, 'docs' => $result], 200);
    }

    /////// For old WP versions
    //save ctpt rate result from checkout page
    public static function saveCtptrateFromCheckout($request)
    {
        $requestData = $request->get_params();
        $data = json_decode($requestData['ctpt_ratedata'], true);

        $savedData = [
            'dadata' => $data['dadata'] ?? [],
            'variant' => $data['variant'] ?? [],
            'selected' => $data['selected'] ?? [],
            'rate_param' => $data['rate_param'] ?? '',
            'Terminal' => $data['Terminal'] ?? [],
            'ctpt_params' => $data['ctpt_params'] ?? [],
        ];
        $cartVariants = new CartVariantsDB();
        $variantId = intval($savedData['variant']['id'] ?? '');
        if ($variantId == 0) return new \WP_REST_Response(['res'=>false], 200);
        $cartVariants->saveCartVariantByCtptrate($variantId, $savedData);

        return new \WP_REST_Response(['res'=>true], 200);
    }


}
