<?php

namespace Ipol\Catapulto\Wordpress\Shipping;

use Ipol\Catapulto\Core\Delivery\Cargo;
use Ipol\Catapulto\Core\Delivery\CargoItem;
use Ipol\Catapulto\Core\Entity\Money;
use Ipol\Catapulto\Wordpress\CatapultoShippingMethod;

class Calculator
{
    const API_DIMENSION_UNIT = 'mm';
    const API_WEIGHT_UNIT = 'g';

    public static function getDefaultGabs():array
    {
        $shippingMethod = new CatapultoShippingMethod();
        $defMode = $shippingMethod->getGabsMode();
        $defWei = wc_get_weight( $shippingMethod->getDefaultWeight(), self::API_WEIGHT_UNIT, 'g' );
        $defHei = wc_get_dimension( $shippingMethod->getDefaultHeight(), self::API_DIMENSION_UNIT, 'mm');
        $defWid = wc_get_dimension( $shippingMethod->getDefaultWidth(), self::API_DIMENSION_UNIT, 'mm');
        $defLen = wc_get_dimension( $shippingMethod->getDefaultLength(), self::API_DIMENSION_UNIT, 'mm');

        return [
            'WEI' => $defWei,
            'HEI' => $defHei,
            'WID' => $defWid,
            'LEN' => $defLen,
            'MODE' => $defMode,
        ];
    }

    /**
     * @return void
     * Для расчета доставки виджетом Катапульто - подготовка данных расчета - параметров грузоместа
     */
    public static function getCurrentCargoDataByCart():array
    {
        if (!WC()->cart) return [];
        $cargoData = self::getCargo( WC()->cart->get_cart() );
        $cargo = $cargoData['cargo'];
        $cargoComment = $cargoData['comment'];

        $dims = $cargo->getDimensions();
        $wei = $cargo->getWeight();

        return array_merge($dims,[
            'CARGO' => $cargo,
            'WEI'=>$wei,
            'TOTAL' => floatval(WC()->cart->get_subtotal()),
            'COMMENT' => $cargoComment
        ]);
    }

    public static function getCurrentCargoDataByOrder(\WC_Order $order):array
    {
        $cargoData = self::getCargo( $order->get_items() );
        $cargo = $cargoData['cargo'];
        $cargoComment = $cargoData['comment'];

        $dims = $cargo->getDimensions();
        $wei = $cargo->getWeight();

        return array_merge($dims,[
            'CARGO' => $cargo,
            'WEI'=>$wei,
            'COMMENT' => $cargoComment
        ]);
    }

    private static function getCargo($items)
    {

        $gabsSettings = self::getDefaultGabs();

        $cargo = new Cargo();
        $cargoComment = '';

        if ($gabsSettings['MODE'] == 'O') {
            $itemsPrices = 0;
            $hasEmpty = false;
            $orderCargo = new Cargo();
            foreach ( $items as $item ) {
                $product = null;
                if ($item instanceof \WC_Order_Item_Product) $product = $item->get_product();
                else $product = $item['data'];

                $wei = intval(round(wc_get_weight(floatval($product->get_weight()), self::API_WEIGHT_UNIT, get_option('woocommerce_weight_unit', 'g') )));
                $hei = intval(round(wc_get_dimension(floatval($product->get_height()), self::API_DIMENSION_UNIT, get_option('woocommerce_dimension_unit', 'mm') )));
                $wid = intval(round(wc_get_dimension(floatval($product->get_width()), self::API_DIMENSION_UNIT, get_option('woocommerce_dimension_unit', 'mm') )));
                $len = intval(round(wc_get_dimension(floatval($product->get_length()), self::API_DIMENSION_UNIT, get_option('woocommerce_dimension_unit', 'mm') )));
                $qty = floatval($item['quantity']);

                $isEmpty = false;
                if ($wei == 0) $isEmpty = true;
                if (
                    ($hei == 0)
                    || ($wid == 0)
                    || ($len == 0)
                ) $isEmpty = true;
                if (!$isEmpty) {
                    $cargoItem = (new CargoItem())
                        ->setHeight($hei)
                        ->setWidth($wid)
                        ->setLength($len)
                        ->setWeight($wei)
                        ->setQuantity($qty)
                        ->setPrice(new Money( floatval($product->get_price()) ))
                        ->setField('name', $product->get_name())
                        ->setField('sku', empty($product->get_sku()) ? '' : $product->get_sku());
                    $orderCargo->add($cargoItem);
                } else $hasEmpty = true;

                $itemsPrices += floatval($product->get_price()) * floatval($item['quantity']);
                $cargoComment .= $product->get_name() . '('.floatval($item['quantity']).');';
            }

            if ($hasEmpty) {
                $dims = $orderCargo->getDimensions();
                $wei = $orderCargo->getWeight();

                $cargoItem = (new CargoItem())
                    ->setHeight( max($gabsSettings['HEI'], $dims['H']) )
                    ->setWidth( max($gabsSettings['WID'], $dims['W']) )
                    ->setLength( max($gabsSettings['LEN'], $dims['L']) )
                    ->setWeight( max($gabsSettings['WEI'], $wei) )
                    ->setQuantity(1)
                    ->setPrice(new Money( $itemsPrices ))
                    ->setField('name', 'order_calculated_gabs')
                    ->setField('sku', '');
                $cargo->add($cargoItem);

            } else $cargo = $orderCargo;

        } else {
            foreach ( $items as $item ) {
                $product = null;
                if ($item instanceof \WC_Order_Item_Product) $product = $item->get_product();
                else $product = $item['data'];
                /** @var WC_Order_Item_Product $product1 */

                /** @var \WC_Product_Simple $product */
                $wei = intval(round(wc_get_weight(floatval($product->get_weight()), self::API_WEIGHT_UNIT, get_option('woocommerce_weight_unit', 'g') )));
                $hei = intval(round(wc_get_dimension(floatval($product->get_height()), self::API_DIMENSION_UNIT, get_option('woocommerce_dimension_unit', 'mm') )));
                $wid = intval(round(wc_get_dimension(floatval($product->get_width()), self::API_DIMENSION_UNIT, get_option('woocommerce_dimension_unit', 'mm') )));
                $len = intval(round(wc_get_dimension(floatval($product->get_length()), self::API_DIMENSION_UNIT, get_option('woocommerce_dimension_unit', 'mm') )));
                $qty = floatval($item['quantity']);

                $isEmpty = false;
                if ($wei == 0) $isEmpty = true;
                if (
                    ($hei == 0)
                    || ($wid == 0)
                    || ($len == 0)
                ) $isEmpty = true;

                if ($isEmpty && ($gabsSettings['MODE'] == 'G')) {
                    $wei = $gabsSettings['WEI'];
                    $len = $gabsSettings['LEN'];
                    $wid = $gabsSettings['WID'];
                    $hei = $gabsSettings['HEI'];
                }

                $cargoComment .= $product->get_name()."(${qty});";
                $cargoItem = (new CargoItem())
                    ->setHeight($hei)
                    ->setWidth($wid)
                    ->setLength($len)
                    ->setWeight($wei)
                    ->setQuantity($qty)
                    ->setPrice(new Money( floatval($product->get_price()) ))
                    ->setField('name', $product->get_name())
                    ->setField('sku', empty($product->get_sku()) ? '' : $product->get_sku());
                $cargo->add($cargoItem);
            }
        }

        return [
            'cargo' => $cargo,
            'comment' => $cargoComment,
        ];
    }

}
