<?php

namespace Ipol\Catapulto\Wordpress\Tools;

use Ipol\Catapulto\Wordpress\CatapultoPlugin;
use Ipol\Catapulto\Wordpress\DB\OperatorsDB;

class TemplateRender
{
    const BASE_ADMIN_URL = '/wp-admin/admin.php?page=';

    private $templateDir;

    public function __construct()
    {
        $this->templateDir = CatapultoPlugin::getMainPluginDir() . DIRECTORY_SEPARATOR . 'templates' . DIRECTORY_SEPARATOR;
    }

    public function renderHtml($content):void
    {
        echo wp_kses($content, [
            'div'=>[ 'class'=>[], 'style'=>[] ],
            'label' => [ 'for' => [], 'class' => [], 'style' => [], ],
            'form' => [ 'action' => [], ],
            'select' => [ 'name' => [], 'class' => [], 'style' => [], 'id' => [], ],
            'option' => [ 'value' => [], ],
            'input' => [ 'name' => [], 'type' => [], 'value' => [], 'class' => [], 'style' => [], 'id' => [], 'size' => [], 'aria-describedby' => [], 'checked' => [], 'k'=>[], 'wpid'=>[], 'readonly'=>[], ],
            'textarea' => [ 'name' => [], ],
            'style' => [],
            'script' => [],
            'button' => [ 'id' => [], 'class' => [], ],
            'br'=>[ 'class'=>[], 'style'=>[], ],
            'ul'=>[ 'class'=>[], 'style'=>[], ],
            'li'=>[ 'class'=>[], 'style'=>[], ],
            'h3'=>[ ],
            'h4'=>[ ],
            'b'=>[ ],
            'hr'=>[ ],
            'img' => [ 'src'=>[], ],
            'p'=>[ 'class'=>[], 'style'=>[], ],
            'a'=>[ 'href'=>[], 'class'=>[], 'style'=>[], ],
            'span' => [ 'class'=>[], 'style'=>[], 'aria-hidden'=>[], ],
            'strong' => [],
            'table' => [ 'class'=>[], 'style'=>[], ],
            'thead' => [],
            'tbody' => [ 'id'=>[], 'data-wp-lists'=>[], ],
            'tfoot' => [],
            'tr' => [ 'id'=>[], 'class'=>[], ],
            'th' => [ 'scope'=>[], 'style' => [], 'class' => [], 'id' => [], 'aria-sort' => [], ],
            'td' => [ 'id' => [], 'class' => [], 'data-colname'=>[],  ],

            'svg' => [ 'viewBox' => [], 'xmlns' => [], 'width' => [], 'height' => [], ],
            'path' => [ 'd' => [], ],
        ]);
    }

    public function renderPaginatorBlock(int $pageNum, int $maxPages, int $recordCount, array $queryParams = []): string
    {
        $firstPageUrl = $lastPageUrl = $nextPageUrl = $prevPageUrl = self::BASE_ADMIN_URL . CatapultoPlugin::PLUGIN_MENU_ID;
        $hiddenFields = '<input type="hidden" name="page" value="'.CatapultoPlugin::PLUGIN_MENU_ID.'" />';
        $qParams = [];

        $isPrevEnabled = true;
        $prevPage = $pageNum - 1;
        if ($prevPage < 1) $isPrevEnabled = false;
        $isNextEnabled = true;
        $nextPage = $pageNum + 1;
        if ($nextPage > $maxPages) $isNextEnabled = false;

        foreach ($queryParams as $key => $value) {
            $qParams[] = implode('=', [$key, $value]);
            $hiddenFields .= '<input type="hidden" name="'.$key.'" value="'.$value.'" />';
        }
        if (!empty($qParams)) {
            $firstPageUrl .= '&' . implode('&', $qParams);
            if ($isPrevEnabled) {
                $p = array_merge($qParams, ['paged='.$prevPage]);
                $prevPageUrl .= '&' . implode('&', $p);
            }
            if ($isNextEnabled) {
                $p1 = array_merge($qParams, ['paged='.$nextPage]);
                $p2 = array_merge($qParams, ['paged='.$maxPages]);
                $nextPageUrl .= '&' . implode('&', $p1);
                $lastPageUrl .= '&' . implode('&', $p2);
            }
        } else {
            $firstPageUrl .= '&paged=1';
            if ($isPrevEnabled) $prevPageUrl .= '&paged='.$prevPage;
            if ($isNextEnabled) {
                $nextPageUrl .= '&paged='.$nextPage;
                $lastPageUrl .= '&paged='.$maxPages;
            }
        }

        $params = [
            'currentPage' => $pageNum,
            'maxPages' => $maxPages,
            'hiddenFields' => $hiddenFields,
            'isOnePage' => ( $maxPages > 1 ) ? '' : 'one-page',
            'text' => Tools::declination($recordCount, ['заказ','заказа','заказов']),
            'firstBtn' => $isPrevEnabled ? '<a class="first-page button" href="'.$firstPageUrl.'"><span class="screen-reader-text">Первая страница</span><span aria-hidden="true">«</span></a>' : '<span class="tablenav-pages-navspan button disabled" aria-hidden="true">«</span>',
            'prevBtn' => $isPrevEnabled ? '<a class="prev-page button" href="'.$prevPageUrl.'"><span class="screen-reader-text">Предыдущая страница</span><span aria-hidden="true">‹</span></a>' : '<span class="tablenav-pages-navspan button disabled" aria-hidden="true">‹</span>',
            'nextBtn' => $isNextEnabled ? '<a class="next-page button" href="'.$nextPageUrl.'"><span class="screen-reader-text">Следующая страница</span><span aria-hidden="true">›</span></a>' : '<span class="tablenav-pages-navspan button disabled" aria-hidden="true">›</span>',
            'lastBtn' => $isNextEnabled ? '<a class="last-page button" href="'.$lastPageUrl.'"><span class="screen-reader-text">Последняя страница</span><span aria-hidden="true">»</span></a>' : '<span class="tablenav-pages-navspan button disabled" aria-hidden="true">»</span>',
        ];

        return $this->render('elements/paginator.tpl', $params);
    }

    public function renderTable(array $columnData, array $records, string $orderField = 'id', string $orderDirection = 'desc')
    {
        $table = '<div class="ctpt_mpg_table_wrapper"><table class="wp-list-table widefat fixed" style="min-width: 3000px;">';
        $head = '<tr><td id="cb" class="manage-column column-cb check-column"><input id="cb-select-all-1" type="checkbox"></td>';
        $body = '';
        foreach ($columnData as $key => $column) {
            $field = '<th scope="col" id="'.$key.'" class="manage-column column-'.$key.'">'.$column['name'].'</th>';
            if (($column['sortable'] ?? false) === true) {
                $link = self::BASE_ADMIN_URL . CatapultoPlugin::PLUGIN_MENU_ID . '&orderby=' . $key . '&order=asc';

                $isSorted = false;
                $sortDirFull = ($orderDirection == 'desc') ? 'aria-sort="descending"' : 'aria-sort="ascending"';
                if ($key == $orderField) {
                    $isSorted = true;
                    $link = self::BASE_ADMIN_URL . CatapultoPlugin::PLUGIN_MENU_ID . '&orderby=' . $key . '&order=';
                    if ($orderDirection == 'desc') $link .= 'asc';
                    else $link .= 'desc';
                }
                $field = '<th scope="col" id="'.$key.'" class="manage-column column-order_number column-primary '.($isSorted ? 'sorted ' . $orderDirection : 'sortable desc').'"'.($isSorted ? ' ' . $sortDirFull : '').'>
                        <a href="'.$link.'">
                            <span>'.$column['name'].'</span>
                            <span class="sorting-indicators">
                                <span class="sorting-indicator asc" aria-hidden="true"></span>
                                <span class="sorting-indicator desc" aria-hidden="true"></span>
                            </span>
                            <span class="screen-reader-text">Сортировать по возрастанию.</span>
                        </a>
                    </th>';
            }
            $head .= $field;
        }
        $head .= '</tr>';

        $operatorsDB = new OperatorsDB();
        $allOperators = $operatorsDB->getAll();
        $icons = [];
        foreach ($allOperators as $operator) {
            if ($operator['op_enabled'] != 'Y') continue;
            $icons[$operator['operator_id']] = $operator['icon'];
        }

        foreach ($records as $record) {
            $row = '<tr id="ctpt_ord_'.$record['id'].'"><th scope="row" class="check-column"><input id="cb-select-'.$record['id'].'" class="ctpt_order_selected" type="checkbox" name="id[]" value="'.$record['id'].'" k="'.$record['key'].'" wpid="'.$record['order_id'].'" /></th>';

            $updated = \DateTime::createFromFormat('Y-m-d H:i:s', $record['updated']);
            $record['updated'] = $updated->format('d.m.Y H:i:s');

            $record['with_insurance'] = ($record['with_insurance'] === '1') ? 'Да' : 'Нет';
            $record['is_pod'] = ($record['is_pod'] === '1') ? 'Да' : 'Нет';

            $documents = unserialize($record['documents']);
            if (!$documents) $documents = [];
            $isDocAll = false;
            $isDocSticker = false;
            $isDocInvoice = false;
            $isPOA = false;
            foreach ($documents as $document) {
                if ($document['title'] === CatapultoPlugin::POA_DOCUMENT_TITLE) $document['key'] = 'POA';
                switch($document['key']) {
                    case 'invoice':
                        $isDocInvoice = true;
                        break;
                    case 'all':
                        $isDocAll = true;
                        break;
                    case 'sticker':
                        $isDocSticker = true;
                        break;
                    case 'POA':
                        $isPOA = true;
                        break;
                }
            }

            $docAll = isset($documents['all']) && !empty($documents['all']);

            foreach ($columnData as $key => $column) {
                $field = '';
                switch ($key) {
                    case 'order_id':
                        $lnkInvoice = $isDocInvoice ? '<span class="hide-if-no-js"><a class="ctpt_print_one" href="#">Печать накладных</a> </span>' : '';
                        $lnkAll = $isDocAll ? '<span class="hide-if-no-js"><a class="ctpt_print_docs_one" href="#">Печать всех документов</a> </span>' : '';
                        $lnkSticker = $isDocSticker ? '<span class="hide-if-no-js"><a class="ctpt_print_stickers_one" href="#">Печать наклеек</a> </span>' : '';
                        $lnkPOA = $isPOA ? '<span class="hide-if-no-js"><a class="ctpt_print_poa_one" href="#">Печать доверенности</a> </span>' : '';
                        $field = '<td class="order_number has-row-actions column-primary column-'.$key.'" data-colname="'.$column['name'].'"><a href="/wp-admin/admin.php?page=wc-orders&action=edit&id='.$record[$key].'" class="order-view" target="_blank"><strong>Заказ #'.$record[$key].'</strong></a><div class="row-actions"><span class="hide-if-no-js"><a class="ctpt_checkstatus_one" href="#">Проверить статус</a> </span>'.$lnkInvoice.$lnkSticker.$lnkPOA.$lnkAll.'<span class="line hide-if-no-js"></span><span class="trash hide-if-no-js"><a class="ctpt_cancelorder_one" href="#">Отменить заявку</a> </span></div></td>';
                        break;
                    case 'tracking_link':
                        $field = '<td class="origin column-'.$key.'" data-colname="'.$column['name'].'"><a href="'.$record[$key].'" class="order-view" target="_blank">'.$record[$key].'</a></td>';
                        break;
                    case 'operator':
                        $field = '<td class="origin column-'.$key.'" data-colname="'.$column['name'].'"><p>'.$record[$key].'</p><img src="'.$icons[$record[$key]].'" /></td>';
                        break;
                    case 'price';
                    case 'sum_to_pay';
                        $field = '<td class="origin column-'.$key.'" data-colname="'.$column['name'].'"><span class="tips"><span class="woocommerce-Price-amount amount">'.$record[$key].'&nbsp;<span class="woocommerce-Price-currencySymbol">₽</span></span></span></td>';
                        break;
                    default:
                        $field = '<td class="origin column-'.$key.'" data-colname="'.$column['name'].'">'.$record[$key].'</td>';
                }
                $row .= $field;
            }
            $body .= $row . '</tr>';
        }

        $table .= '<thead>'.$head.'</thead><tbody>'.$body.'</tbody><tfoot>'.$head.'</tfoot></table></div>';
        return $table;
    }

    public function render(string $template, array $contentData, bool $isShow = false):?string
    {
        $templateFile = $this->templateDir . $template;

        if (!file_exists($templateFile) || !is_readable($templateFile)) return '';
        $content = file_get_contents($templateFile);
        foreach ($contentData as $key => $value) $content = preg_replace('/#' . $key . '#/', $value, $content);

        if ($isShow) {
            $this->renderHtml($content);
            return null;
        }
        return $content;
    }

}
